#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars import _CustomIslamicHolidays
from holidays.calendars.gregorian import JAN, FEB, MAR, APR, MAY, JUN, JUL, AUG, SEP, OCT, NOV, DEC
from holidays.constants import GOVERNMENT, PUBLIC, WORKDAY
from holidays.groups import (
    ChristianHolidays,
    InternationalHolidays,
    IslamicHolidays,
    StaticHolidays,
)
from holidays.holiday_base import HolidayBase


class TimorLeste(
    HolidayBase, ChristianHolidays, InternationalHolidays, IslamicHolidays, StaticHolidays
):
    """Timor Leste holidays.

    References:
        * [2005 Law](https://web.archive.org/web/20240813193212/https://mj.gov.tl/jornal/lawsTL/RDTL-Law/RDTL-Laws/Law-2005-10.pdf)
        * [2016 Amendment](https://web.archive.org/web/20250414152148/https://timor-leste.gov.tl/?p=14494&lang=en)
        * [2022](https://web.archive.org/web/20240802065911/https://timor-leste.gov.tl/?p=30266&lang=en)
        * [2023 (en_US)](https://web.archive.org/web/20250414071435/https://timor-leste.gov.tl/?lang=en&p=31750)
        * [2023 (pt_PT)](https://web.archive.org/web/20250416013718/https://timor-leste.gov.tl/?lang=pt&p=31750)
        * [2023 (tet)](https://web.archive.org/web/20250415232148/https://timor-leste.gov.tl/?p=31750&lang=tp)
        * [2024](https://web.archive.org/web/20250421164147/https://timor-leste.gov.tl/?p=35833&lang=en)
        * [2025](https://web.archive.org/web/20250416022640/https://timor-leste.gov.tl/?lang=en&p=41492)

    Limitations:
        * Exact Islamic holidays dates are only available for 2011-2025; the rest are estimates.
    """

    country = "TL"
    supported_categories = (GOVERNMENT, PUBLIC, WORKDAY)
    default_language = "pt_TL"
    # %s (estimated).
    estimated_label = tr("%s (aproximada)")
    supported_languages = ("en_TL", "en_US", "pt_TL", "tet", "th")
    # Law No. 10/2005 Of 10 August, Public Holidays and Official Commemorative Dates.
    start_year = 2006

    def __init__(self, *args, islamic_show_estimated: bool = True, **kwargs):
        """
        Args:
            islamic_show_estimated:
                Whether to add "estimated" label to Islamic holidays name
                if holiday date is estimated.
        """
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        IslamicHolidays.__init__(
            self, cls=TimorLesteIslamicHolidays, show_estimated=islamic_show_estimated
        )
        StaticHolidays.__init__(self, TimorLesteStaticHolidays)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # Fixed Date Public Holidays.

        # New Year's Day.
        self._add_new_years_day(tr("Dia de Ano Novo"))

        # Dia dos Veteranos.
        # First appeared in 2017.

        if self._year >= 2017:
            # Veteran's Day.
            self._add_holiday_mar_3(tr("Dia dos Veteranos"))

        # Good Friday.
        self._add_good_friday(tr("Sexta-Feira Santa"))

        # International Worker's Day.
        self._add_labor_day(tr("Dia Mundial do Trabalhador"))

        # Restoration of Independence Day.
        self._add_holiday_may_20(tr("Dia da Restauração da Independência"))

        # Corpus Christi.
        self._add_corpus_christi_day(tr("Festa do Corpo de Deus"))

        # Popular Consultation Day.
        self._add_holiday_aug_30(tr("Dia da Consulta Popular"))

        # All Saints' Day.
        self._add_all_saints_day(tr("Dia de Todos os Santos"))

        # All Souls' Day.
        self._add_all_souls_day(tr("Dia de Todos os Fiéis Defuntos"))

        # Dia Nacional da Mulher.
        # Originally classed as "Commemorative Date" only, reclassified in 2023.

        if self._year >= 2023:
            # National Women's Day.
            self._add_holiday_nov_3(tr("Dia Nacional da Mulher"))

        # National Youth Day.
        self._add_holiday_nov_12(tr("Dia Nacional da Juventude"))

        # Proclamation of Independence Day.
        self._add_holiday_nov_28(tr("Dia da Proclamação da Independência"))

        # Dia da Memória
        # Created to replaced the original National Heroes Day in 2017.

        if self._year >= 2017:
            # Memorial Day.
            self._add_holiday_dec_7(tr("Dia da Memória"))

        self._add_immaculate_conception_day(
            # Day of Our Lady of Immaculate Conception and Timor-Leste Patroness.
            tr("Dia de Nossa Senhora da Imaculada Conceição, padroeira de Timor-Leste")
        )

        # Christmas Day.
        self._add_christmas_day(tr("Dia de Natal"))

        # Dia dos Heróis Nacionais.
        # Moved to Dec 31 in 2017.

        # National Heroes Day.
        name = tr("Dia dos Heróis Nacionais")
        if self._year >= 2017:
            self._add_holiday_dec_31(name)
        else:
            self._add_holiday_dec_7(name)

        # Eid al-Fitr.
        self._add_eid_al_fitr_day(tr("Idul Fitri"))

        # Eid al-Adha.
        self._add_eid_al_adha_day(tr("Idul Adha"))

    def _populate_workday_holidays(self):
        # Ash Wednesday.
        self._add_ash_wednesday(tr("Quarta-Feira de Cinzas"))

        # Maundy Thursday.
        self._add_holy_thursday(tr("Quinta-Feira Santa"))

        # The Day of Ascension of Jesus Christ into Heaven.
        self._add_ascension_thursday(tr("Dia da Ascensão de Jesus Cristo ao Céu"))

        # International Children's Day.
        self._add_childrens_day(tr("Dia Mundial da Criança"))

        self._add_holiday_aug_20(
            # Day of the Armed Forces for the National Liberation of Timor-Leste (FALINTIL).
            tr("Dia das Forças Armadas de Libertação Nacional de Timor-Leste (FALINTIL)")
        )

        # Dia Nacional da Mulher.
        # Originally classed as "Commemorative Date" only, reclassified in 2023.
        # Prior to reclassification, this is usually only observed as half-day holiday. i.e.
        # 2010:
        # https://web.archive.org/web/20250427130447/https://timor-leste.gov.tl/?p=4183&lang=en
        # 2011:
        # https://web.archive.org/web/20250414065353/https://timor-leste.gov.tl/?p=5979&lang=en
        if self._year <= 2022:
            # National Women's Day.
            self._add_holiday_nov_3(tr("Dia Nacional da Mulher"))

        # International Human Rights Day.
        self._add_holiday_dec_10(tr("Dia Mundial dos Direitos Humanos"))


class TL(TimorLeste):
    pass


class TLS(TimorLeste):
    pass


class TimorLesteIslamicHolidays(_CustomIslamicHolidays):
    EID_AL_ADHA_DATES_CONFIRMED_YEARS = (2011, 2025)
    EID_AL_ADHA_DATES = {
        2011: (NOV, 7),
        2015: (SEP, 24),
        2016: (SEP, 18),
        2021: (JUL, 19),
        2023: (JUN, 29),
        2024: (JUN, 17),
    }

    EID_AL_FITR_DATES_CONFIRMED_YEARS = (2011, 2025)
    EID_AL_FITR_DATES = {
        2011: (AUG, 31),
        2012: (AUG, 20),
        2016: (JUL, 7),
        2017: (JUN, 26),
        2019: (JUN, 6),
        2023: (APR, 22),
        2025: (MAR, 31),
    }


class TimorLesteStaticHolidays:
    # Special Holidays.

    # National Holidays (Special).
    special_national_holidays = tr("Feriados Nacionais (Especiais)")

    # Presidential Election Day.
    presidential_election = tr("Dia da Eleição Presidencial")

    # Parliamentary Election Day.
    parliamentary_election = tr("Dia de Eleições Parlamentares")

    # Local Election Day.
    local_election = tr("Dia de eleições locais")

    # Centenary of the Revolt of Dom Boaventura.
    dom_boaventura_centenary = tr("Centenário da Revolta de Dom Boaventura")

    # Funeral Ceremonies of Fernando 'La Sama' de Araújo.
    la_sama_funeral = tr("Cerimónias Fúnebres de Fernando 'La Sama' de Araújo")

    # 20th Anniversary Celebrations of the Popular Consultation.
    popular_consultation_20th = tr("Celebrações do 20.º Aniversário da Consulta Popular")

    # 25th Anniversary Celebrations of the Popular Consultation.
    popular_consultation_25th = tr("Celebrações do 25.º Aniversário da Consulta Popular")

    # Visit of His Holiness Pope Francis to Timor-Leste.
    pope_francis_visit = tr("Visita de Sua Santidade o Papa Francisco a Timor-Leste")

    special_government_holidays = {
        2010: (
            # https://web.archive.org/web/20250427130447/https://timor-leste.gov.tl/?p=4183&lang=en
            (NOV, 3, special_national_holidays),
            # https://web.archive.org/web/20250414065444/https://timor-leste.gov.tl/?p=4437&lang=en
            (DEC, 24, special_national_holidays),
            (DEC, 31, special_national_holidays),
        ),
        2011: (
            # https://web.archive.org/web/20250414065414/https://timor-leste.gov.tl/?p=5496&lang=en
            (AUG, 15, special_national_holidays),
            # https://web.archive.org/web/20250414065353/https://timor-leste.gov.tl/?p=5979&lang=en
            (NOV, 3, special_national_holidays),
            # https://web.archive.org/web/20250427130448/https://timor-leste.gov.tl/?p=6264&lang=en
            (DEC, 26, special_national_holidays),
        ),
        2012: (
            # https://web.archive.org/web/20250427130448/https://timor-leste.gov.tl/?p=6264&lang=en
            (JAN, 2, special_national_holidays),
            # https://web.archive.org/web/20250414065422/https://timor-leste.gov.tl/?p=6347&lang=en
            (JAN, 23, special_national_holidays),
            # https://web.archive.org/web/20250414065452/https://timor-leste.gov.tl/?p=6471&lang=en
            (FEB, 22, special_national_holidays),
            # https://web.archive.org/web/20250427130503/https://timor-leste.gov.tl/?p=6621&lang=en
            (MAR, 16, presidential_election),
            # https://web.archive.org/web/20250414065432/https://timor-leste.gov.tl/?p=6760&lang=en
            (APR, 16, presidential_election),
            (APR, 17, presidential_election),
            # https://web.archive.org/web/20250427130515/https://timor-leste.gov.tl/?p=7035&lang=en
            (JUL, 6, parliamentary_election),
            # https://web.archive.org/web/20250427130516/https://timor-leste.gov.tl/?p=7046&lang=en
            (JUL, 9, parliamentary_election),
            # https://web.archive.org/web/20250414065639/https://timor-leste.gov.tl/?p=7474&lang=en
            (NOV, 27, dom_boaventura_centenary),
            (NOV, 29, dom_boaventura_centenary),
            # https://web.archive.org/web/20250414065642/http://timor-leste.gov.tl/?p=7550&lang=en
            (DEC, 24, special_national_holidays),
            (DEC, 26, special_national_holidays),
            (DEC, 31, special_national_holidays),
        ),
        2013: (
            # https://web.archive.org/web/20250427130528/https://timor-leste.gov.tl/?p=7715&lang=en
            (FEB, 13, special_national_holidays),
            # https://web.archive.org/web/20250427130538/https://timor-leste.gov.tl/?p=7918&lang=en
            (MAR, 28, special_national_holidays),
            (APR, 1, special_national_holidays),
            # https://web.archive.org/web/20250414065612/https://timor-leste.gov.tl/?p=8664&lang=en
            (AUG, 20, special_national_holidays),
            # https://web.archive.org/web/20250414065748/https://timor-leste.gov.tl/?p=9392&lang=en
            (NOV, 29, special_national_holidays),
            # https://web.archive.org/web/20250414065744/http://timor-leste.gov.tl/?p=9475&lang=en
            (DEC, 24, special_national_holidays),
            (DEC, 26, special_national_holidays),
            (DEC, 31, special_national_holidays),
        ),
        2014: (
            # https://web.archive.org/web/20250427130548/https://timor-leste.gov.tl/?p=9759&lang=en
            (MAR, 5, special_national_holidays),
            # https://web.archive.org/web/20210826174115/http://timor-leste.gov.tl/?p=9964&lang=en
            (APR, 17, special_national_holidays),
            (APR, 21, special_national_holidays),
            # https://web.archive.org/web/20250427130153/http://timor-leste.gov.tl/?p=10294&lang=en
            (JUL, 22, special_national_holidays),
            (JUL, 23, special_national_holidays),
            # https://web.archive.org/web/20250414065707/https://timor-leste.gov.tl/?p=10524&lang=en
            (AUG, 15, special_national_holidays),
            (AUG, 20, special_national_holidays),
            # https://web.archive.org/web/20250427130202/https://timor-leste.gov.tl/?p=11036&lang=en
            (DEC, 24, special_national_holidays),
            (DEC, 26, special_national_holidays),
            (DEC, 31, special_national_holidays),
        ),
        2015: (
            # https://web.archive.org/web/20250427130202/https://timor-leste.gov.tl/?p=11036&lang=en
            (JAN, 2, special_national_holidays),
            # https://web.archive.org/web/20250414065729/https://timor-leste.gov.tl/?p=11247&lang=en
            (FEB, 18, special_national_holidays),
            # https://web.archive.org/web/20250414065741/https://timor-leste.gov.tl/?p=11544&lang=en
            (APR, 2, special_national_holidays),
            # https://web.archive.org/web/20250414065752/https://timor-leste.gov.tl/?p=11966&lang=en
            (MAY, 13, special_national_holidays),
            # https://web.archive.org/web/20230624130431/http://timor-leste.gov.tl/?p=12246&lang=en
            (JUN, 5, la_sama_funeral),
            # https://web.archive.org/web/20250414065950/https://timor-leste.gov.tl/?p=13105&lang=en
            (AUG, 20, special_national_holidays),
            # https://web.archive.org/web/20250414065829/https://timor-leste.gov.tl/?p=14271&lang=en
            (DEC, 24, special_national_holidays),
            (DEC, 31, special_national_holidays),
        ),
        2016: (
            # https://web.archive.org/web/20240816060741/https://timor-leste.gov.tl/?p=14482&lang=en
            (FEB, 10, special_national_holidays),
            # https://web.archive.org/web/20250414070020/https://timor-leste.gov.tl/?p=14827&lang=en
            (MAR, 24, special_national_holidays),
            # https://web.archive.org/web/20250414065859/https://timor-leste.gov.tl/?p=15740&lang=en
            (JUL, 6, special_national_holidays),
            # https://web.archive.org/web/20250414070052/https://timor-leste.gov.tl/?p=16626&lang=en
            (NOV, 3, special_national_holidays),
            # https://web.archive.org/web/20250427130231/https://timor-leste.gov.tl/?p=16998&lang=en
            (DEC, 26, special_national_holidays),
        ),
        2017: (
            # https://web.archive.org/web/20250427130231/https://timor-leste.gov.tl/?p=16998&lang=en
            (JAN, 2, special_national_holidays),
            # https://web.archive.org/web/20250427130238/https://timor-leste.gov.tl/?p=17428&lang=en
            (MAR, 1, special_national_holidays),
            # https://web.archive.org/web/20250427130247/https://timor-leste.gov.tl/?p=17548&lang=en
            (MAR, 20, presidential_election),
            (MAR, 21, presidential_election),
            # https://web.archive.org/web/20250414070116/https://timor-leste.gov.tl/?p=17698&lang=en
            (APR, 13, special_national_holidays),
            # https://web.archive.org/web/20250427130254/https://timor-leste.gov.tl/?p=19189&lang=en
            (DEC, 26, special_national_holidays),
        ),
        2018: (
            # https://web.archive.org/web/20250427130254/https://timor-leste.gov.tl/?p=19189&lang=en
            (JAN, 2, special_national_holidays),
            # https://web.archive.org/web/20240813145627/https://timor-leste.gov.tl/?p=19411&lang=en
            (FEB, 14, special_national_holidays),
            # https://web.archive.org/web/20250414070149/https://timor-leste.gov.tl/?p=19452&lang=en
            (FEB, 16, special_national_holidays),
            # https://web.archive.org/web/20240813145438/https://timor-leste.gov.tl/?p=19693&lang=en
            (MAR, 29, special_national_holidays),
            # https://web.archive.org/web/20250414070232/https://timor-leste.gov.tl/?p=20199&lang=en
            (AUG, 22, special_national_holidays),
        ),
        2019: (
            # https://web.archive.org/web/20250414070216/https://timor-leste.gov.tl/?p=21116&lang=en
            (FEB, 5, special_national_holidays),
            # https://web.archive.org/web/20250414070241/https://timor-leste.gov.tl/?p=21207&lang=en
            (MAR, 6, special_national_holidays),
            # https://web.archive.org/web/20240812151629/https://timor-leste.gov.tl/?p=21607&lang=en
            (APR, 18, special_national_holidays),
            # https://web.archive.org/web/20250427130325/https://timor-leste.gov.tl/?p=22642&lang=en
            (AUG, 12, special_national_holidays),
            # https://web.archive.org/web/20250414070318/https://timor-leste.gov.tl/?p=22681&lang=en
            (AUG, 20, special_national_holidays),
            # https://web.archive.org/web/20240812044011/https://timor-leste.gov.tl/?p=22701&lang=en
            (AUG, 26, popular_consultation_20th),
            (AUG, 27, popular_consultation_20th),
            (AUG, 28, popular_consultation_20th),
            (AUG, 29, popular_consultation_20th),
            # https://web.archive.org/web/20250414070227/https://timor-leste.gov.tl/?p=23277&lang=en
            (OCT, 31, special_national_holidays),
            # https://web.archive.org/web/20250414070409/https://timor-leste.gov.tl/?p=23417&lang=en
            (DEC, 24, special_national_holidays),
            (DEC, 26, special_national_holidays),
            (DEC, 30, special_national_holidays),
        ),
        2020: (
            # https://web.archive.org/web/20250414070409/https://timor-leste.gov.tl/?p=23417&lang=en
            (JAN, 2, special_national_holidays),
            # https://web.archive.org/web/20250427130330/https://timor-leste.gov.tl/?p=23607&lang=en
            (FEB, 26, special_national_holidays),
            # https://web.archive.org/web/20250427130347/https://timor-leste.gov.tl/?p=25455&lang=en
            (AUG, 20, special_national_holidays),
            # https://web.archive.org/web/20250414070335/https://timor-leste.gov.tl/?p=25502&lang=en
            (AUG, 31, special_national_holidays),
            # https://web.archive.org/web/20250414070433/https://timor-leste.gov.tl/?p=26030&lang=en
            (NOV, 3, special_national_holidays),
            # https://web.archive.org/web/20250427130521/https://timor-leste.gov.tl/?p=26365&lang=en
            (DEC, 24, special_national_holidays),
        ),
        2021: (
            # https://web.archive.org/web/20250414070431/https://timor-leste.gov.tl/?p=26865&lang=en
            (FEB, 12, special_national_holidays),
            # https://web.archive.org/web/20250414070435/https://timor-leste.gov.tl/?p=26896&lang=en
            (FEB, 17, special_national_holidays),
            # https://web.archive.org/web/20250414070527/https://timor-leste.gov.tl/?p=29682&lang=en
            (NOV, 3, special_national_holidays),
        ),
        2022: (
            # https://web.archive.org/web/20250414070535/https://timor-leste.gov.tl/?p=30029&lang=en
            (FEB, 1, special_national_holidays),
            # https://web.archive.org/web/20250414070533/https://timor-leste.gov.tl/?p=30194&lang=en
            (MAR, 2, special_national_holidays),
            # https://web.archive.org/web/20250414070535/https://timor-leste.gov.tl/?p=30254&lang=en
            (MAR, 18, presidential_election),
            # https://web.archive.org/web/20250414070614/https://timor-leste.gov.tl/?p=30429&lang=en
            (APR, 14, special_national_holidays),
            (APR, 18, presidential_election),
            (APR, 19, presidential_election),
            (APR, 20, presidential_election),
            # https://web.archive.org/web/20250414070557/https://timor-leste.gov.tl/?p=31107&lang=en
            (AUG, 29, special_national_holidays),
            # https://web.archive.org/web/20250414070537/https://timor-leste.gov.tl/?p=31152&lang=en
            (SEP, 6, special_national_holidays),
            # https://web.archive.org/web/20250414070629/https://timor-leste.gov.tl/?p=31404&lang=en
            (OCT, 31, special_national_holidays),
            # https://web.archive.org/web/20250427130438/https://timor-leste.gov.tl/?p=31574&lang=en
            (DEC, 9, special_national_holidays),
            # https://web.archive.org/web/20250414070644/https://timor-leste.gov.tl/?p=31633&lang=en
            (DEC, 26, special_national_holidays),
        ),
        2023: (
            # https://web.archive.org/web/20250414070658/https://timor-leste.gov.tl/?p=31641&lang=en
            (JAN, 2, special_national_holidays),
            # https://web.archive.org/web/20250414070652/https://timor-leste.gov.tl/?p=31798&lang=en
            (JAN, 23, special_national_holidays),
            # https://web.archive.org/web/20250414070730/https://timor-leste.gov.tl/?p=32191&lang=en
            (FEB, 22, special_national_holidays),
            # https://web.archive.org/web/20250414070922/https://timor-leste.gov.tl/?p=32561&lang=en
            (APR, 6, special_national_holidays),
            (APR, 10, special_national_holidays),
            # https://web.archive.org/web/20250414070723/https://timor-leste.gov.tl/?p=32590&lang=en
            (APR, 20, special_national_holidays),
            (APR, 21, special_national_holidays),
            # https://web.archive.org/web/20250414070814/https://timor-leste.gov.tl/?p=32617&lang=en
            (MAY, 19, parliamentary_election),
            (MAY, 22, parliamentary_election),
            # https://web.archive.org/web/20250414035027/https://timor-leste.gov.tl/?p=34792&lang=en
            (OCT, 27, local_election),
            # https://web.archive.org/web/20250414070835/https://timor-leste.gov.tl/?p=35060&lang=en
            (NOV, 13, local_election),
            # https://web.archive.org/web/20250429134659/https://timor-leste.gov.tl/?p=35627&lang=en&
            (DEC, 26, special_national_holidays),
        ),
        2024: (
            # https://web.archive.org/web/20250417085916/https://timor-leste.gov.tl/?p=35627&lang=en
            (JAN, 2, special_national_holidays),
            # https://web.archive.org/web/20250414070914/https://timor-leste.gov.tl/?p=36002&lang=en
            (FEB, 14, special_national_holidays),
            # https://web.archive.org/web/20250420193703/https://timor-leste.gov.tl/?p=36859&lang=en
            (MAR, 28, special_national_holidays),
            # https://web.archive.org/web/20250414121144/https://timor-leste.gov.tl/?p=39062&lang=en
            (AUG, 28, popular_consultation_25th),
            (AUG, 29, popular_consultation_25th),
            # https://web.archive.org/web/20250414070900/https://timor-leste.gov.tl/?p=39068&lang=en
            (SEP, 9, pope_francis_visit),
            (SEP, 10, pope_francis_visit),
            (SEP, 11, pope_francis_visit),
            # https://web.archive.org/web/20250421064400/https://timor-leste.gov.tl/?p=40592&lang=en
            (OCT, 31, special_national_holidays),
            # https://web.archive.org/web/20250414071012/https://timor-leste.gov.tl/?p=40955&lang=en
            (NOV, 29, special_national_holidays),
            # https://web.archive.org/web/20250414071025/https://timor-leste.gov.tl/?p=41325&lang=en
            (DEC, 24, special_national_holidays),
        ),
        2025: (
            # https://web.archive.org/web/20250414070915/https://timor-leste.gov.tl/?p=41361&lang=en
            (JAN, 2, special_national_holidays),
            # https://web.archive.org/web/20250414070923/https://timor-leste.gov.tl/?p=41592&lang=en
            (JAN, 29, special_national_holidays),
            # https://web.archive.org/web/20250414070934/https://timor-leste.gov.tl/?p=42076&lang=en
            (MAR, 5, special_national_holidays),
        ),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

import warnings
from unittest import TestCase

from holidays.countries.norway import Norway
from tests.common import CommonCountryTests, SundayHolidays


class TestNorway(CommonCountryTests, SundayHolidays, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Norway)

    def setUp(self):
        super().setUp()
        warnings.simplefilter("ignore", category=DeprecationWarning)

    def test_subdiv_deprecation(self):
        self.assertDeprecatedSubdivisions("This subdivision is deprecated and will be removed")

    def test_new_years(self):
        self.assertHoliday("1901-01-01", "2017-01-01", "2023-01-01")

    def test_easter(self):
        self.assertHoliday(
            "2000-04-20",
            "2000-04-21",
            "2000-04-23",
            "2000-04-24",
            "2010-04-01",
            "2010-04-02",
            "2010-04-04",
            "2010-04-05",
            "2021-04-01",
            "2021-04-02",
            "2021-04-04",
            "2021-04-05",
            "2024-03-28",
            "2024-03-29",
            "2024-03-31",
            "2024-04-01",
        )

    def test_workers_day(self):
        self.assertHoliday("1947-05-01", "2017-05-01", "2023-05-01")
        self.assertNoHoliday("1946-05-01")
        self.assertNoHolidayName("Arbeidernes dag", Norway(years=1946))

    def test_constitution_day(self):
        self.assertHoliday("1947-05-17", "2017-05-17", "2023-05-17")
        self.assertNoHoliday("1946-05-17")
        self.assertNoHolidayName("Grunnlovsdag", Norway(years=1946))

    def test_pentecost(self):
        self.assertHoliday(
            "2000-06-11",
            "2000-06-12",
            "2010-05-23",
            "2010-05-24",
            "2023-05-28",
            "2023-05-29",
        )

    def test_christmas(self):
        self.assertHoliday(
            "1901-12-25",
            "1901-12-26",
            "2016-12-25",
            "2016-12-26",
        )

    def test_sundays(self):
        self.assertSundays(Norway)  # Sundays are considered holidays in Norway.

    def test_not_holiday(self):
        # TODO: Add more dates that are often confused for being a holiday.

        # Sundays in Norway are considered holidays,
        # so make sure none of these are actually Sundays.
        self.assertNoHoliday(
            "2017-02-06",
            "2017-02-07",
            "2017-02-08",
            "2017-02-09",
            "2017-02-10",
            "2001-12-24",
            "2001-05-16",
            "2001-05-18",
            "1999-12-31",
            "2016-12-31",
            "2016-12-27",
            "2016-12-28",
        )

    def test_2022(self):
        self.assertHolidaysInYear(
            2022,
            ("2022-01-01", "Første nyttårsdag"),
            ("2022-04-14", "Skjærtorsdag"),
            ("2022-04-15", "Langfredag"),
            ("2022-04-17", "Første påskedag"),
            ("2022-04-18", "Andre påskedag"),
            ("2022-05-01", "Arbeidernes dag"),
            ("2022-05-17", "Grunnlovsdag"),
            ("2022-05-26", "Kristi himmelfartsdag"),
            ("2022-06-05", "Første pinsedag"),
            ("2022-06-06", "Andre pinsedag"),
            ("2022-12-25", "Første juledag"),
            ("2022-12-26", "Andre juledag"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "Første nyttårsdag"),
            ("2022-04-14", "Skjærtorsdag"),
            ("2022-04-15", "Langfredag"),
            ("2022-04-17", "Første påskedag"),
            ("2022-04-18", "Andre påskedag"),
            ("2022-05-01", "Arbeidernes dag"),
            ("2022-05-17", "Grunnlovsdag"),
            ("2022-05-26", "Kristi himmelfartsdag"),
            ("2022-06-05", "Første pinsedag"),
            ("2022-06-06", "Andre pinsedag"),
            ("2022-12-25", "Første juledag"),
            ("2022-12-26", "Andre juledag"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-04-14", "Maundy Thursday"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-17", "Easter Sunday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-05-01", "Labor Day"),
            ("2022-05-17", "Constitution Day"),
            ("2022-05-26", "Ascension Day"),
            ("2022-06-05", "Whit Sunday"),
            ("2022-06-06", "Whit Monday"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Second Day of Christmas"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2022-01-01", "Новий рік"),
            ("2022-04-14", "Великий четвер"),
            ("2022-04-15", "Страсна пʼятниця"),
            ("2022-04-17", "Великдень"),
            ("2022-04-18", "Великодній понеділок"),
            ("2022-05-01", "День праці"),
            ("2022-05-17", "День Конституції"),
            ("2022-05-26", "Вознесіння Господнє"),
            ("2022-06-05", "Трійця"),
            ("2022-06-06", "День Святого Духа"),
            ("2022-12-25", "Різдво Христове"),
            ("2022-12-26", "Другий день Різдва"),
        )

    def test_l10n_th(self):
        self.assertLocalizedHolidays(
            "th",
            ("2022-01-01", "วันขึ้นปีใหม่"),
            ("2022-04-14", "วันพฤหัสศักดิ์สิทธิ์"),
            ("2022-04-15", "วันศุกร์ประเสริฐ"),
            ("2022-04-17", "วันอาทิตย์อีสเตอร์"),
            ("2022-04-18", "วันจันทร์อีสเตอร์"),
            ("2022-05-01", "วันแรงงาน"),
            ("2022-05-17", "วันรัฐธรรมนูญ"),
            ("2022-05-26", "วันสมโภชพระเยซูเจ้าเสด็จขึ้นสวรรค์"),
            ("2022-06-05", "วันสมโภชพระจิตเจ้า"),
            ("2022-06-06", "วันจันทร์หลังวันสมโภชพระจิตเจ้า"),
            ("2022-12-25", "วันคริสต์มาสวันแรก"),
            ("2022-12-26", "วันคริสต์มาสวันที่สอง"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars import _CustomHinduHolidays, _CustomIslamicHolidays
from holidays.calendars.gregorian import JAN, FEB, MAR, APR, MAY, JUN, JUL, SEP, OCT, NOV, DEC
from holidays.groups import (
    ChristianHolidays,
    HinduCalendarHolidays,
    IslamicHolidays,
    StaticHolidays,
)
from holidays.observed_holiday_base import ObservedHolidayBase, SAT_TO_NONE, SUN_TO_NONE


class NationalStockExchangeOfIndia(
    ObservedHolidayBase, HinduCalendarHolidays, ChristianHolidays, IslamicHolidays, StaticHolidays
):
    """National Stock Exchange of India (NSE) holidays.

    References:
        * <https://web.archive.org/web/20250821175252/https://www.nseindia.com/resources/exchange-communication-circulars>

    Historical data:
        * [2001](https://archive.org/details/nsearchives.nseindia.comcontentcircularscmtr2189.wri)
        * [2002](https://archive.org/details/cmtr3058)
        * [2003](https://web.archive.org/web/20250904042405/https://nsearchives.nseindia.com/content/circulars/cmtr3809.htm)
        * [2004](https://archive.org/details/nsearchives.nseindia.comcontentcircularscmtr4645.htm)
        * [2005](https://web.archive.org/web/20250904043234/https://nsearchives.nseindia.com/content/circulars/cmtr5633.htm)
        * [2006](https://archive.org/details/nsearchives.nseindia.comcontentcircularscmtr6946.htm)
        * [2007](https://archive.org/details/nsearchives.nseindia.comcontentcircularscmtr8182.pdf)
        * [2008](https://archive.org/details/nsearchives.nseindia.comcontentcircularscmtr9908.htm)
        * [2009](https://archive.org/details/nsearchives.nseindia.comcontentcircularscmtr11733.htm)
        * [2010](https://archive.org/details/nsearchives.nseindia.comcontentcircularscmtr13713.pdf)
        * [2011](https://archive.org/details/nsearchives.nseindia.comcontentcircularscmtr16348.pdf)
        * [2012](https://archive.org/details/nsearchives.nseindia.comcontentcircularscmtr19539.pdf)
        * [2013](https://archive.org/details/nsearchives.nseindia.comcontentcircularscmtr22317.pdf)
        * [2014](https://archive.org/details/nsearchives.nseindia.comcontentcircularscmtr25326.pdf)
        * [2015](https://archive.org/details/nsearchives.nseindia.comcontentcircularscmtr28337.pdf)
        * [2016](https://web.archive.org/web/20250903152854/https://nsearchives.nseindia.com/content/circulars/CMTR31297.pdf)
        * [2017](https://web.archive.org/web/20250903152829/https://nsearchives.nseindia.com/content/circulars/CMTR33746.pdf)
        * [2018](https://web.archive.org/web/20250903152307/https://nsearchives.nseindia.com/content/circulars/CMTR36475.pdf)
        * [2019](https://archive.org/details/nsearchives.nseindia.comcontentcircularscmtr39612.pdf)
        * [2020](https://web.archive.org/web/20250903152140/https://nsearchives.nseindia.com/content/circulars/CMTR42877.pdf)
        * [2021](https://web.archive.org/web/20250903152041/https://nsearchives.nseindia.com/content/circulars/CMTR46623.pdf)
        * [2022](https://web.archive.org/web/20250821071611/https://nsearchives.nseindia.com/content/circulars/CMTR50560.pdf)
        * [2023](https://web.archive.org/web/20250821071635/https://nsearchives.nseindia.com/content/circulars/CMTR54757.pdf)
        * [2024](https://web.archive.org/web/20250821071650/https://nsearchives.nseindia.com/content/circulars/CMTR59722.pdf)
        * [2025](https://web.archive.org/web/20250624132016/https://nsearchives.nseindia.com/content/circulars/CMTR65587.pdf)
    """

    market = "XNSE"
    default_language = "en_IN"
    # %s (estimated).
    estimated_label = tr("%s (estimated)")
    supported_languages = ("en_IN", "en_US", "gu", "hi", "mr")
    start_year = 2001

    def __init__(self, *args, islamic_show_estimated: bool = True, **kwargs):
        """
        Args:
            islamic_show_estimated:
                Whether to add "estimated" label to Islamic holidays name
                if holiday date is estimated.

        In India, the dates of the Islamic calendar usually fall a day later than
        the corresponding dates in the Umm al-Qura calendar.
        """
        ChristianHolidays.__init__(self)
        HinduCalendarHolidays.__init__(self, cls=NationalStockExchangeOfIndiaHinduHolidays)
        IslamicHolidays.__init__(
            self,
            cls=NationalStockExchangeOfIndiaIslamicHolidays,
            show_estimated=islamic_show_estimated,
            calendar_delta_days=+1,
        )
        StaticHolidays.__init__(self, cls=NationalStockExchangeOfIndiaStaticHolidays)
        kwargs.setdefault("observed_rule", SAT_TO_NONE + SUN_TO_NONE)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # Republic Day.
        self._move_holiday(self._add_holiday_jan_26(tr("Republic Day")))

        # Good Friday.
        self._add_good_friday(tr("Good Friday"))

        # Dr. B. R. Ambedkar Jayanti.
        self._move_holiday(self._add_holiday_apr_14(tr("Dr. Baba Saheb Ambedkar Jayanti")))

        self._move_holiday(
            self._add_holiday_may_1(
                # May Day.
                tr("May Day")
                if 2010 <= self._year <= 2014
                # Maharashtra Day.
                else tr("Maharashtra Day")
            )
        )

        # Independence Day.
        self._move_holiday(self._add_holiday_aug_15(tr("Independence Day")))

        # Gandhi Jayanti.
        self._move_holiday(self._add_holiday_oct_2(tr("Mahatma Gandhi Jayanti")))

        # Christmas Day.
        self._move_holiday(self._add_christmas_day(tr("Christmas Day")))

        # Hindu Calendar Holidays.

        if self._year >= 2007:
            # Maha Shivaratri.
            self._move_holiday(self._add_maha_shivaratri(tr("Maha Shivaratri")))

        # Holi.
        self._move_holiday(self._add_holi(tr("Holi")))

        if self._year >= 2006:
            # Ram Navami.
            self._move_holiday(self._add_ram_navami(tr("Ram Navami")))

            # Mahavir Jayanti.
            self._move_holiday(self._add_mahavir_jayanti(tr("Mahavir Jayanti")))

        # Ganesh Chaturthi.
        self._move_holiday(self._add_ganesh_chaturthi(tr("Ganesh Chaturthi")))

        # Dussehra.
        self._move_holiday(self._add_dussehra(tr("Dussehra")))

        # Diwali Lakshmi Puja.
        self._move_holiday(self._add_diwali_india(tr("Diwali Laxmi Pujan")))

        if self._year <= 2002 or self._year >= 2011:
            # Diwali Balipratipada.
            self._move_holiday(self._add_govardhan_puja(tr("Diwali Balipratipada")))

        # Guru Nanak Jayanti.
        self._move_holiday(self._add_guru_nanak_jayanti(tr("Guru Nanak Jayanti")))

        if 2003 <= self._year <= 2010:
            # Bhai Dooj.
            self._move_holiday(self._add_bhai_dooj(tr("Bhau Bhij")))

        if 2006 <= self._year <= 2009:
            # Buddha Purnima.
            self._move_holiday(self._add_buddha_purnima(tr("Buddha Purnima")))

        # Islamic Calendar Holidays.

        # Ashura.
        for dt in self._add_ashura_day(tr("Muharram")):
            self._move_holiday(dt)

        if 2006 <= self._year <= 2009:
            # Prophet's Birthday.
            for dt in self._add_mawlid_day(tr("Id-E-Milad-Un-Nabi")):
                self._move_holiday(dt)

        # Eid al-Fitr.
        for dt in self._add_eid_al_fitr_day(tr("Id-Ul-Fitr (Ramadan Eid)")):
            self._move_holiday(dt)

        # Eid al-Adha.
        for dt in self._add_eid_al_adha_day(tr("Bakri Id")):
            self._move_holiday(dt)


class XNSE(NationalStockExchangeOfIndia):
    pass


class NSE(NationalStockExchangeOfIndia):
    pass


class NationalStockExchangeOfIndiaHinduHolidays(_CustomHinduHolidays):
    BUDDHA_PURNIMA_DATES = {
        2008: (MAY, 19),
    }

    DIWALI_INDIA_DATES = {
        2024: (NOV, 1),
        2025: (OCT, 21),
    }

    DUSSEHRA_DATES = {
        2003: (OCT, 4),
        2018: (OCT, 18),
    }

    GOVARDHAN_PUJA_DATES = {
        2001: (NOV, 16),
        2002: (NOV, 6),
        2003: (OCT, 25),
        2006: (OCT, 23),
        2020: (NOV, 16),
        2022: (OCT, 26),
        2023: (NOV, 14),
    }

    HOLI_DATES = {
        2023: (MAR, 7),
    }

    MAHAVIR_JAYANTI_DATES = {
        2010: (MAR, 28),
        2016: (APR, 19),
    }

    RAM_NAVAMI_DATES = {
        2007: (MAR, 27),
    }


class NationalStockExchangeOfIndiaIslamicHolidays(_CustomIslamicHolidays):
    ASHURA_DATES_CONFIRMED_YEARS = (2001, 2025)
    ASHURA_DATES = {
        2003: (MAR, 15),
        2006: (FEB, 9),
        2008: (JAN, 19),
        2015: (OCT, 23),
        2018: (SEP, 20),
    }

    EID_AL_ADHA_DATES_CONFIRMED_YEARS = (2001, 2025)
    EID_AL_ADHA_DATES = {
        2003: (FEB, 13),
        2005: (JAN, 21),
        2006: (JAN, 11),
        2007: ((JAN, 1), (DEC, 21)),
        2009: (NOV, 27),
        2014: (OCT, 6),
        2015: (SEP, 25),
        2016: (SEP, 13),
        2023: (JUN, 28),
    }

    EID_AL_FITR_DATES_CONFIRMED_YEARS = (2001, 2025)
    EID_AL_FITR_DATES = {
        2002: (DEC, 7),
        2005: (NOV, 5),
        2006: (OCT, 25),
        2010: (SEP, 10),
        2016: (JUL, 6),
        2021: (MAY, 13),
    }

    MAWLID_DATES_CONFIRMED_YEARS = (2006, 2009)
    MAWLID_DATES = {
        2008: (MAR, 20),
    }


class NationalStockExchangeOfIndiaStaticHolidays:
    """National Stock Exchange of India (NSE) special holidays."""

    special_public_holidays = {
        # New Year's Day.
        2010: (JAN, 1, tr("New Year")),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars.gregorian import _timedelta, FRI, SAT
from holidays.constants import OPTIONAL, PUBLIC, SCHOOL
from holidays.groups import HebrewCalendarHolidays
from holidays.observed_holiday_base import (
    ObservedHolidayBase,
    MON_TO_NEXT_TUE,
    THU_TO_PREV_WED,
    FRI_TO_PREV_WED,
    FRI_TO_PREV_THU,
    SAT_TO_PREV_THU,
    SAT_TO_NEXT_SUN,
    SUN_TO_NEXT_MON,
)


class Israel(ObservedHolidayBase, HebrewCalendarHolidays):
    """Israel holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Israel>
        * <https://web.archive.org/web/20190923042619/https://www.knesset.gov.il/laws/special/heb/jerusalem_day_law.htm>
    """

    country = "IL"
    default_language = "he"
    # %s (observed).
    observed_label = tr("%s (נצפה)")
    supported_categories = (OPTIONAL, PUBLIC, SCHOOL)
    supported_languages = ("en_US", "he", "th", "uk")
    weekend = {FRI, SAT}
    start_year = 1948

    def __init__(self, *args, **kwargs):
        HebrewCalendarHolidays.__init__(self)
        kwargs.setdefault("observed_rule", FRI_TO_PREV_THU + SAT_TO_PREV_THU)
        super().__init__(*args, **kwargs)

    def _add_observed(self, dt, name, rule):
        is_observed, _ = super()._add_observed(dt, name, rule=rule)
        if not is_observed:
            self._add_holiday(name, dt)

    def _populate_public_holidays(self):
        # Rosh Hashanah (New Year).
        self._add_rosh_hashanah(tr("ראש השנה"), range(2))

        # Yom Kippur (Day of Atonement).
        self._add_yom_kippur(tr("יום כיפור"))

        # Sukkot (Feast of Tabernacles).
        self._add_sukkot(tr("סוכות"))
        # Simchat Torah / Shemini Atzeret.
        self._add_sukkot(tr("שמחת תורה/שמיני עצרת"), +7)

        # Pesach (Passover).
        self._add_passover(tr("פסח"))
        # Shvi'i shel Pesach (Seventh day of Passover)
        self._add_passover(tr("שביעי של פסח"), +6)

        rule = FRI_TO_PREV_THU + SAT_TO_PREV_THU
        if self._year >= 2004:
            rule += MON_TO_NEXT_TUE
        self._add_observed(
            self._hebrew_calendar.israel_independence_date(self._year),
            # Yom Ha-Atzmaut (Independence Day).
            tr("יום העצמאות"),
            rule,
        )

        # Shavuot.
        self._add_shavuot(tr("שבועות"))

    def _populate_optional_holidays(self):
        # Chol HaMoed Sukkot (Feast of Tabernacles holiday).
        self._add_sukkot(tr("חול המועד סוכות"), range(1, 6))

        if self._year >= 2008:
            # Sigd.
            self._add_yom_kippur(tr("סיגד"), +49)

        # Purim.
        self._add_purim(tr("פורים"))

        # Chol HaMoed Pesach (Passover holiday).
        self._add_passover(tr("חול המועד פסח"), range(1, 6))

        if self._year >= 1963:
            rule = THU_TO_PREV_WED + FRI_TO_PREV_WED
            if self._year >= 2004:
                rule += SUN_TO_NEXT_MON
            self._add_observed(
                _timedelta(self._hebrew_calendar.israel_independence_date(self._year), -1),
                # Yom Hazikaron (Fallen Soldiers and Victims of Terrorism Remembrance Day).
                tr("יום הזיכרון לחללי מערכות ישראל ונפגעי פעולות האיבה"),
                rule,
            )

        if self._year >= 1998:
            # Yom Yerushalayim (Jerusalem Day).
            self._add_lag_baomer(tr("יום ירושלים"), +10)

        self._add_observed(
            self._hebrew_calendar.tisha_bav_date(self._year),
            # Tisha B'Av (Tisha B'Av, fast).
            tr("תשעה באב"),
            SAT_TO_NEXT_SUN,
        )

    def _populate_school_holidays(self):
        # Chol HaMoed Sukkot (Feast of Tabernacles holiday).
        self._add_sukkot(tr("חול המועד סוכות"), range(1, 6))

        # Hanukkah.
        self._add_hanukkah(tr("חנוכה"), range(8))

        self._add_observed(
            _timedelta(self._hebrew_calendar.purim_date(self._year), -1),
            # Ta`anit Ester (Fast of Esther).
            tr("תענית אסתר"),
            SAT_TO_PREV_THU,
        )

        # Purim.
        self._add_purim(tr("פורים"))

        # Chol HaMoed Pesach (Passover holiday).
        self._add_passover(tr("חול המועד פסח"), range(1, 6))

        # Lag Ba'omer (Lag BaOmer).
        self._add_lag_baomer(tr('ל"ג בעומר'))


class IL(Israel):
    pass


class ISR(Israel):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.vatican_city import VaticanCity
from tests.common import CommonCountryTests


class TestVaticanCity(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(VaticanCity, years=range(1929, 2050))

    def test_solemnity_of_mary_mother_of_god(self):
        self.assertHolidayName(
            "Solennità di Maria Santissima Madre di Dio",
            (f"{year}-01-01" for year in range(1929, 2050)),
        )

    def test_epiphany(self):
        self.assertHolidayName(
            "Epifania del Signore", (f"{year}-01-06" for year in range(1929, 2050))
        )

    def test_anniversary_of_the_foundation_of_vatican_city(self):
        self.assertHolidayName(
            "Anniversario della istituzione dello Stato della Città del Vaticano",
            (f"{year}-02-11" for year in range(1929, 2050)),
        )

    def test_anniversary_election_of_holy_father(self):
        name = "Anniversario dell'Elezione del Santo Padre"
        self.assertHolidayName(name, (f"{year}-02-06" for year in range(1929, 1940)))
        self.assertHolidayName(name, (f"{year}-03-02" for year in range(1940, 1959)))
        self.assertHolidayName(name, (f"{year}-10-28" for year in range(1959, 1963)))
        self.assertHolidayName(name, (f"{year}-06-21" for year in range(1964, 1979)))
        self.assertHolidayName(name, (f"{year}-10-16" for year in range(1979, 2005)))
        self.assertHolidayName(name, (f"{year}-04-19" for year in range(2006, 2013)))
        self.assertHolidayName(name, (f"{year}-03-13" for year in range(2014, 2026)))
        self.assertHolidayName(name, (f"{year}-05-08" for year in range(2026, 2050)))
        self.assertNoHolidayName(name, 1963, 2005, 2013)

    def test_name_day_of_holy_father(self):
        name = "Onomastico del Santo Padre"
        self.assertHolidayName(name, (f"{year}-11-04" for year in range(1978, 2005)))
        self.assertHolidayName(name, (f"{year}-03-19" for year in range(2006, 2013)))
        self.assertHolidayName(name, (f"{year}-04-23" for year in range(2013, 2026)))
        self.assertHolidayName(name, (f"{year}-09-17" for year in range(2025, 2050)))
        self.assertNoHolidayName(name, range(1929, 1978), 2005)

    def test_saint_josephs_day(self):
        self.assertHolidayName("San Giuseppe", (f"{year}-03-19" for year in range(1929, 2050)))

    def test_maundy_thursday(self):
        name = "Giovedì Santo"
        self.assertHolidayName(
            name,
            "2019-04-18",
            "2020-04-09",
            "2021-04-01",
            "2022-04-14",
            "2023-04-06",
            "2024-03-28",
            "2025-04-17",
        )
        self.assertHolidayName(name, range(1929, 2050))

    def test_good_friday(self):
        name = "Venerdì Santo"
        self.assertHolidayName(
            name,
            "2019-04-19",
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, range(1929, 2050))

    def test_holy_saturday(self):
        name = "Sabato Santo"
        self.assertHolidayName(
            name,
            "2019-04-20",
            "2020-04-11",
            "2021-04-03",
            "2022-04-16",
            "2023-04-08",
            "2024-03-30",
            "2025-04-19",
        )
        self.assertHolidayName(name, range(1929, 2050))

    def test_easter_sunday(self):
        name = "Pasqua di Resurrezione"
        self.assertHolidayName(
            name,
            "2019-04-21",
            "2020-04-12",
            "2021-04-04",
            "2022-04-17",
            "2023-04-09",
            "2024-03-31",
            "2025-04-20",
        )
        self.assertHolidayName(name, range(1929, 2050))

    def test_easter_monday(self):
        name = "Lunedì dell'Angelo"
        self.assertHolidayName(
            name,
            "2019-04-22",
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, range(1929, 2050))

    def test_easter_tuesday(self):
        name = "Martedì in Albis"
        self.assertHolidayName(
            name,
            "2019-04-23",
            "2020-04-14",
            "2021-04-06",
            "2022-04-19",
            "2023-04-11",
            "2024-04-02",
            "2025-04-22",
        )
        self.assertHolidayName(name, range(1929, 2050))

    def test_ascension_day(self):
        name = "Ascensione del Signore"
        self.assertHolidayName(
            name,
            "2019-05-30",
            "2020-05-21",
            "2021-05-13",
            "2022-05-26",
            "2023-05-18",
            "2024-05-09",
            "2025-05-29",
        )
        self.assertHolidayName(name, range(1929, 2050))

    def test_corpus_domini(self):
        name = "Corpus Domini"
        self.assertHolidayName(
            name,
            "2019-06-20",
            "2020-06-11",
            "2021-06-03",
            "2022-06-16",
            "2023-06-08",
            "2024-05-30",
            "2025-06-19",
        )
        self.assertHolidayName(name, range(1929, 2050))

    def test_saint_joseph_the_workers_day(self):
        name = "San Giuseppe Artigiano"
        self.assertHolidayName(name, (f"{year}-05-01" for year in range(1955, 2050)))
        self.assertNoHolidayName(name, range(1929, 1955))

    def test_solemnity_of_pentecost(self):
        name = "Solennità della Pentecoste"
        self.assertHolidayName(
            name,
            "2019-06-09",
            "2020-05-31",
            "2021-05-23",
            "2022-06-05",
            "2023-05-28",
            "2024-05-19",
            "2025-06-08",
        )
        self.assertHolidayName(name, range(1929, 2050))

    def test_solemnity_of_holy_trinity(self):
        name = "Solennità della Santissima Trinità"
        self.assertHolidayName(
            name,
            "2019-06-16",
            "2020-06-07",
            "2021-05-30",
            "2022-06-12",
            "2023-06-04",
            "2024-05-26",
            "2025-06-15",
        )
        self.assertHolidayName(name, range(1929, 2050))

    def test_saints_peter_and_paul_day(self):
        self.assertHolidayName(
            "Santi Pietro e Paolo", (f"{year}-06-29" for year in range(1929, 2050))
        )

    def test_assumption_day(self):
        self.assertHolidayName(
            "Vigilia dell'Assunzione di Maria Santissima",
            (f"{year}-08-14" for year in range(1929, 2050)),
        )
        self.assertHolidayName(
            "Assunzione di Maria Santissima", (f"{year}-08-15" for year in range(1929, 2050))
        )
        self.assertHolidayName(
            "Giorno Successivo all'Assunzione di Maria Santissima",
            (f"{year}-08-16" for year in range(1929, 2050)),
        )

    def test_all_saints_day(self):
        self.assertHolidayName("Tutti i Santi", (f"{year}-11-01" for year in range(1929, 2050)))

    def test_all_souls_day(self):
        self.assertHolidayName(
            "Tutti i Fedeli Defunti", (f"{year}-11-02" for year in range(1929, 2050))
        )

    def test_immaculate_conception(self):
        self.assertHolidayName(
            "Immacolata Concezione", (f"{year}-12-08" for year in range(1929, 2050))
        )

    def test_christmas_days(self):
        self.assertHolidayName(
            "Vigilia di Natale", (f"{year}-12-24" for year in range(1929, 2050))
        )
        self.assertHolidayName("Natale", (f"{year}-12-25" for year in range(1929, 2050)))
        self.assertHolidayName("Santo Stefano", (f"{year}-12-26" for year in range(1929, 2050)))
        self.assertHolidayName("San Giovanni", (f"{year}-12-27" for year in range(1929, 2050)))

    def test_last_day_of_the_year(self):
        self.assertHolidayName(
            "Ultimo giorno dell'anno", (f"{year}-12-31" for year in range(1929, 2050))
        )

    def test_2020(self):
        # https://www.vaticanstate.va/images/pdf/CALENDARIO_2020.pdf
        self.assertHolidaysInYear(
            2020,
            ("2020-01-01", "Solennità di Maria Santissima Madre di Dio"),
            ("2020-01-06", "Epifania del Signore"),
            ("2020-02-11", "Anniversario della istituzione dello Stato della Città del Vaticano"),
            ("2020-03-13", "Anniversario dell'Elezione del Santo Padre"),
            ("2020-03-19", "San Giuseppe"),
            ("2020-04-09", "Giovedì Santo"),
            ("2020-04-10", "Venerdì Santo"),
            ("2020-04-11", "Sabato Santo"),
            ("2020-04-12", "Pasqua di Resurrezione"),
            ("2020-04-13", "Lunedì dell'Angelo"),
            ("2020-04-14", "Martedì in Albis"),
            ("2020-04-23", "Onomastico del Santo Padre"),
            ("2020-05-01", "San Giuseppe Artigiano"),
            ("2020-05-21", "Ascensione del Signore"),
            ("2020-05-31", "Solennità della Pentecoste"),
            ("2020-06-07", "Solennità della Santissima Trinità"),
            ("2020-06-11", "Corpus Domini"),
            ("2020-06-29", "Santi Pietro e Paolo"),
            ("2020-08-14", "Vigilia dell'Assunzione di Maria Santissima"),
            ("2020-08-15", "Assunzione di Maria Santissima"),
            ("2020-08-16", "Giorno Successivo all'Assunzione di Maria Santissima"),
            ("2020-11-01", "Tutti i Santi"),
            ("2020-11-02", "Tutti i Fedeli Defunti"),
            ("2020-12-08", "Immacolata Concezione"),
            ("2020-12-24", "Vigilia di Natale"),
            ("2020-12-25", "Natale"),
            ("2020-12-26", "Santo Stefano"),
            ("2020-12-27", "San Giovanni"),
            ("2020-12-31", "Ultimo giorno dell'anno"),
        )

    def test_2021(self):
        # https://www.farmaciavaticana.va/images/pdf/calendario_2021.pdf
        self.assertHolidaysInYear(
            2021,
            ("2021-01-01", "Solennità di Maria Santissima Madre di Dio"),
            ("2021-01-06", "Epifania del Signore"),
            ("2021-02-11", "Anniversario della istituzione dello Stato della Città del Vaticano"),
            ("2021-03-13", "Anniversario dell'Elezione del Santo Padre"),
            ("2021-03-19", "San Giuseppe"),
            ("2021-04-01", "Giovedì Santo"),
            ("2021-04-02", "Venerdì Santo"),
            ("2021-04-03", "Sabato Santo"),
            ("2021-04-04", "Pasqua di Resurrezione"),
            ("2021-04-05", "Lunedì dell'Angelo"),
            ("2021-04-06", "Martedì in Albis"),
            ("2021-04-23", "Onomastico del Santo Padre"),
            ("2021-05-01", "San Giuseppe Artigiano"),
            ("2021-05-13", "Ascensione del Signore"),
            ("2021-05-23", "Solennità della Pentecoste"),
            ("2021-05-30", "Solennità della Santissima Trinità"),
            ("2021-06-03", "Corpus Domini"),
            ("2021-06-29", "Santi Pietro e Paolo"),
            ("2021-08-14", "Vigilia dell'Assunzione di Maria Santissima"),
            ("2021-08-15", "Assunzione di Maria Santissima"),
            ("2021-08-16", "Giorno Successivo all'Assunzione di Maria Santissima"),
            ("2021-11-01", "Tutti i Santi"),
            ("2021-11-02", "Tutti i Fedeli Defunti"),
            ("2021-12-08", "Immacolata Concezione"),
            ("2021-12-24", "Vigilia di Natale"),
            ("2021-12-25", "Natale"),
            ("2021-12-26", "Santo Stefano"),
            ("2021-12-27", "San Giovanni"),
            ("2021-12-31", "Ultimo giorno dell'anno"),
        )

    def test_2022(self):
        # https://www.farmaciavaticana.va/images/pdf/calendario_2022.pdf
        self.assertHolidaysInYear(
            2022,
            ("2022-01-01", "Solennità di Maria Santissima Madre di Dio"),
            ("2022-01-06", "Epifania del Signore"),
            ("2022-02-11", "Anniversario della istituzione dello Stato della Città del Vaticano"),
            ("2022-03-13", "Anniversario dell'Elezione del Santo Padre"),
            ("2022-03-19", "San Giuseppe"),
            ("2022-04-14", "Giovedì Santo"),
            ("2022-04-15", "Venerdì Santo"),
            ("2022-04-16", "Sabato Santo"),
            ("2022-04-17", "Pasqua di Resurrezione"),
            ("2022-04-18", "Lunedì dell'Angelo"),
            ("2022-04-19", "Martedì in Albis"),
            ("2022-04-23", "Onomastico del Santo Padre"),
            ("2022-05-01", "San Giuseppe Artigiano"),
            ("2022-05-26", "Ascensione del Signore"),
            ("2022-06-05", "Solennità della Pentecoste"),
            ("2022-06-12", "Solennità della Santissima Trinità"),
            ("2022-06-16", "Corpus Domini"),
            ("2022-06-29", "Santi Pietro e Paolo"),
            ("2022-08-14", "Vigilia dell'Assunzione di Maria Santissima"),
            ("2022-08-15", "Assunzione di Maria Santissima"),
            ("2022-08-16", "Giorno Successivo all'Assunzione di Maria Santissima"),
            ("2022-11-01", "Tutti i Santi"),
            ("2022-11-02", "Tutti i Fedeli Defunti"),
            ("2022-12-08", "Immacolata Concezione"),
            ("2022-12-24", "Vigilia di Natale"),
            ("2022-12-25", "Natale"),
            ("2022-12-26", "Santo Stefano"),
            ("2022-12-27", "San Giovanni"),
            ("2022-12-31", "Ultimo giorno dell'anno"),
        )

    def test_2023(self):
        # https://www.farmaciavaticana.va/images/pdf/calendario_2023.pdf
        self.assertHolidaysInYear(
            2023,
            ("2023-01-01", "Solennità di Maria Santissima Madre di Dio"),
            ("2023-01-06", "Epifania del Signore"),
            ("2023-02-11", "Anniversario della istituzione dello Stato della Città del Vaticano"),
            ("2023-03-13", "Anniversario dell'Elezione del Santo Padre"),
            ("2023-03-19", "San Giuseppe"),
            ("2023-04-06", "Giovedì Santo"),
            ("2023-04-07", "Venerdì Santo"),
            ("2023-04-08", "Sabato Santo"),
            ("2023-04-09", "Pasqua di Resurrezione"),
            ("2023-04-10", "Lunedì dell'Angelo"),
            ("2023-04-11", "Martedì in Albis"),
            ("2023-04-23", "Onomastico del Santo Padre"),
            ("2023-05-01", "San Giuseppe Artigiano"),
            ("2023-05-18", "Ascensione del Signore"),
            ("2023-05-28", "Solennità della Pentecoste"),
            ("2023-06-04", "Solennità della Santissima Trinità"),
            ("2023-06-08", "Corpus Domini"),
            ("2023-06-29", "Santi Pietro e Paolo"),
            ("2023-08-14", "Vigilia dell'Assunzione di Maria Santissima"),
            ("2023-08-15", "Assunzione di Maria Santissima"),
            ("2023-08-16", "Giorno Successivo all'Assunzione di Maria Santissima"),
            ("2023-11-01", "Tutti i Santi"),
            ("2023-11-02", "Tutti i Fedeli Defunti"),
            ("2023-12-08", "Immacolata Concezione"),
            ("2023-12-24", "Vigilia di Natale"),
            ("2023-12-25", "Natale"),
            ("2023-12-26", "Santo Stefano"),
            ("2023-12-27", "San Giovanni"),
            ("2023-12-31", "Ultimo giorno dell'anno"),
        )

    def test_2024(self):
        # https://www.farmaciavaticana.va/media/attachments/2024/01/02/calendario_2024.pdf
        self.assertHolidaysInYear(
            2024,
            ("2024-01-01", "Solennità di Maria Santissima Madre di Dio"),
            ("2024-01-06", "Epifania del Signore"),
            ("2024-02-11", "Anniversario della istituzione dello Stato della Città del Vaticano"),
            ("2024-03-13", "Anniversario dell'Elezione del Santo Padre"),
            ("2024-03-19", "San Giuseppe"),
            ("2024-03-28", "Giovedì Santo"),
            ("2024-03-29", "Venerdì Santo"),
            ("2024-03-30", "Sabato Santo"),
            ("2024-03-31", "Pasqua di Resurrezione"),
            ("2024-04-01", "Lunedì dell'Angelo"),
            ("2024-04-02", "Martedì in Albis"),
            ("2024-04-23", "Onomastico del Santo Padre"),
            ("2024-05-01", "San Giuseppe Artigiano"),
            ("2024-05-09", "Ascensione del Signore"),
            ("2024-05-19", "Solennità della Pentecoste"),
            ("2024-05-26", "Solennità della Santissima Trinità"),
            ("2024-05-30", "Corpus Domini"),
            ("2024-06-29", "Santi Pietro e Paolo"),
            ("2024-08-14", "Vigilia dell'Assunzione di Maria Santissima"),
            ("2024-08-15", "Assunzione di Maria Santissima"),
            ("2024-08-16", "Giorno Successivo all'Assunzione di Maria Santissima"),
            ("2024-11-01", "Tutti i Santi"),
            ("2024-11-02", "Tutti i Fedeli Defunti"),
            ("2024-12-08", "Immacolata Concezione"),
            ("2024-12-24", "Vigilia di Natale"),
            ("2024-12-25", "Natale"),
            ("2024-12-26", "Santo Stefano"),
            ("2024-12-27", "San Giovanni"),
            ("2024-12-31", "Ultimo giorno dell'anno"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2025-01-01", "Solennità di Maria Santissima Madre di Dio"),
            ("2025-01-06", "Epifania del Signore"),
            ("2025-02-11", "Anniversario della istituzione dello Stato della Città del Vaticano"),
            ("2025-03-13", "Anniversario dell'Elezione del Santo Padre"),
            ("2025-03-19", "San Giuseppe"),
            ("2025-04-17", "Giovedì Santo"),
            ("2025-04-18", "Venerdì Santo"),
            ("2025-04-19", "Sabato Santo"),
            ("2025-04-20", "Pasqua di Resurrezione"),
            ("2025-04-21", "Lunedì dell'Angelo"),
            ("2025-04-22", "Martedì in Albis"),
            ("2025-04-23", "Onomastico del Santo Padre"),
            ("2025-05-01", "San Giuseppe Artigiano"),
            ("2025-05-29", "Ascensione del Signore"),
            ("2025-06-08", "Solennità della Pentecoste"),
            ("2025-06-15", "Solennità della Santissima Trinità"),
            ("2025-06-19", "Corpus Domini"),
            ("2025-06-29", "Santi Pietro e Paolo"),
            ("2025-08-14", "Vigilia dell'Assunzione di Maria Santissima"),
            ("2025-08-15", "Assunzione di Maria Santissima"),
            ("2025-08-16", "Giorno Successivo all'Assunzione di Maria Santissima"),
            ("2025-09-17", "Onomastico del Santo Padre"),
            ("2025-11-01", "Tutti i Santi"),
            ("2025-11-02", "Tutti i Fedeli Defunti"),
            ("2025-12-08", "Immacolata Concezione"),
            ("2025-12-24", "Vigilia di Natale"),
            ("2025-12-25", "Natale"),
            ("2025-12-26", "Santo Stefano"),
            ("2025-12-27", "San Giovanni"),
            ("2025-12-31", "Ultimo giorno dell'anno"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2025-01-01", "Solemnity of Mary, Mother of God"),
            ("2025-01-06", "Epiphany"),
            ("2025-02-11", "Anniversary of the Foundation of Vatican City"),
            ("2025-03-13", "Anniversary of the Election of the Holy Father"),
            ("2025-03-19", "Saint Joseph's Day"),
            ("2025-04-17", "Maundy Thursday"),
            ("2025-04-18", "Good Friday"),
            ("2025-04-19", "Holy Saturday"),
            ("2025-04-20", "Easter Sunday"),
            ("2025-04-21", "Easter Monday"),
            ("2025-04-22", "Easter Tuesday"),
            ("2025-04-23", "Name Day of the Holy Father"),
            ("2025-05-01", "Saint Joseph the Worker's Day"),
            ("2025-05-29", "Ascension Day"),
            ("2025-06-08", "Solemnity of Pentecost"),
            ("2025-06-15", "Solemnity of Holy Trinity"),
            ("2025-06-19", "Corpus Domini"),
            ("2025-06-29", "Saints Peter and Paul's Day"),
            ("2025-08-14", "Day Before Assumption of Mary"),
            ("2025-08-15", "Assumption of Mary Day"),
            ("2025-08-16", "Day After Assumption of Mary"),
            ("2025-09-17", "Name Day of the Holy Father"),
            ("2025-11-01", "All Saints' Day"),
            ("2025-11-02", "All Souls' Day"),
            ("2025-12-08", "Immaculate Conception"),
            ("2025-12-24", "Christmas Eve"),
            ("2025-12-25", "Christmas Day"),
            ("2025-12-26", "Saint Stephen's Day"),
            ("2025-12-27", "Saint John the Evangelist's Day"),
            ("2025-12-31", "Last Day of the Year"),
        )

    def test_l10n_th(self):
        self.assertLocalizedHolidays(
            "th",
            ("2025-01-01", "วันสมโภชพระนางมารีอา พระชนนีพระเป็นเจ้า"),
            ("2025-01-06", "วันสมโภชพระคริสต์แสดงองค์"),
            ("2025-02-11", "วันครบรอบการสถาปนานครรัฐวาติกัน"),
            ("2025-03-13", "วันคล้ายวันเลือกตั้งสมเด็จพระสันตะปาปา"),
            ("2025-03-19", "วันสมโภชนักบุญโยเซฟ"),
            ("2025-04-17", "วันพฤหัสศักดิ์สิทธิ์"),
            ("2025-04-18", "วันศุกร์ประเสริฐ"),
            ("2025-04-19", "วันเสาร์ศักดิ์สิทธิ์"),
            ("2025-04-20", "วันอาทิตย์อีสเตอร์"),
            ("2025-04-21", "วันจันทร์อีสเตอร์"),
            ("2025-04-22", "วันอังคารอีสเตอร์"),
            ("2025-04-23", "วันฉลองพระนามเดิมสมเด็จพระสันตะปาปา"),
            ("2025-05-01", "วันฉลองนักบุญโยเซฟ กรรมกร"),
            ("2025-05-29", "วันสมโภชพระเยซูเจ้าเสด็จขึ้นสวรรค์"),
            ("2025-06-08", "วันสมโภชพระจิตเจ้า"),
            ("2025-06-15", "วันสมโภชพระตรีเอกภาพ"),
            ("2025-06-19", "วันสมโภชพระคริสตวรกาย"),
            ("2025-06-29", "วันสมโภชนักบุญเปโตรและเปาโล"),
            ("2025-08-14", "วันก่อนวันสมโภชแม่พระรับเกียรติยกขึ้นสวรรค์"),
            ("2025-08-15", "วันสมโภชแม่พระรับเกียรติยกขึ้นสวรรค์"),
            ("2025-08-16", "วันหลังวันสมโภชแม่พระรับเกียรติยกขึ้นสวรรค์"),
            ("2025-09-17", "วันฉลองพระนามเดิมสมเด็จพระสันตะปาปา"),
            ("2025-11-01", "วันสมโภชนักบุญทั้งหลาย"),
            ("2025-11-02", "วันภาวนาอุทิศแด่ผู้ล่วงลับ"),
            ("2025-12-08", "วันสมโภชแม่พระผู้ปฏิสนธินิรมล"),
            ("2025-12-24", "วันคริสต์มาสอีฟ"),
            ("2025-12-25", "วันคริสต์มาส"),
            ("2025-12-26", "วันสมโภชนักบุญสเตเฟน"),
            ("2025-12-27", "วันสมโภชนักบุญยอห์น"),
            ("2025-12-31", "วันสิ้นปี"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.guyana import Guyana
from tests.common import CommonCountryTests


class TestGuyana(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Guyana)

    def test_special_holidays(self):
        self.assertHolidayName("Public Holiday", "2020-03-02")

    def test_new_years_day(self):
        name = "New Year's Day"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        dt = (
            "2011-01-03",
            "2012-01-02",
            "2017-01-02",
            "2022-01-03",
            "2023-01-02",
        )
        self.assertHolidayName(f"{name} (observed)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_independence_day(self):
        name = "Independence Day"
        self.assertHolidayName(
            name,
            (
                f"{year}-05-26"
                for year in (*range(self.start_year, 1970), *range(2016, self.end_year))
            ),
        )
        self.assertNoHolidayName(name, range(1970, 2016))

    def test_republic_day(self):
        name = "Republic Day"
        self.assertHolidayName(name, (f"{year}-02-23" for year in range(1970, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1970))

    def test_good_friday(self):
        name = "Good Friday"
        self.assertHolidayName(
            name,
            "2019-04-19",
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "Easter Monday"
        self.assertHolidayName(
            name,
            "2019-04-22",
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_labour_day(self):
        name = "Labour Day"
        self.assertHolidayName(name, (f"{year}-05-01" for year in self.full_range))
        dt = (
            "2011-05-02",
            "2016-05-02",
            "2022-05-02",
        )
        self.assertHolidayName(f"{name} (observed)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_arrival_day(self):
        name = "Arrival Day"
        self.assertHolidayName(name, (f"{year}-05-05" for year in range(2019, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2019))
        dt = (
            "2019-05-06",
            "2024-05-06",
        )
        self.assertHolidayName(f"{name} (observed)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_caricom_day(self):
        name = "CARICOM Day"
        self.assertHolidayName(
            name,
            "2021-07-05",
            "2022-07-04",
            "2023-07-03",
            "2024-07-01",
            "2025-07-07",
        )
        self.assertHolidayName(name, range(2016, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 2016))

    def test_commonwealth_day(self):
        name = "Commonwealth Day"
        self.assertHolidayName(
            name,
            "2011-08-01",
            "2012-08-06",
            "2013-08-05",
            "2014-08-04",
            "2015-08-03",
        )
        self.assertHolidayName(name, range(self.start_year, 2016))
        self.assertNoHolidayName(name, range(2016, self.end_year))

    def test_emancipation_day(self):
        name = "Emancipation Day"
        self.assertHolidayName(name, (f"{year}-08-01" for year in range(2016, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2016))
        dt = (
            "2021-08-02",
            "2027-08-02",
        )
        self.assertHolidayName(f"{name} (observed)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_christmas_day(self):
        name = "Christmas Day"
        self.assertHolidayName(name, (f"{year}-12-25" for year in self.full_range))
        dt = (
            "2010-12-27",
            "2011-12-27",
            "2016-12-27",
            "2021-12-27",
            "2022-12-27",
        )
        self.assertHolidayName(f"{name} (observed)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_day_after_christmas(self):
        name = "Day after Christmas"
        self.assertHolidayName(name, (f"{year}-12-26" for year in self.full_range))
        dt = (
            "2009-12-28",
            "2010-12-28",
            "2015-12-28",
            "2020-12-28",
            "2021-12-28",
        )
        self.assertHolidayName(f"{name} (observed)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_phagwah_day(self):
        name = "Phagwah"
        self.assertHolidayName(
            name,
            "2020-03-10",
            "2021-03-28",
            "2022-03-18",
            "2023-03-07",
            "2024-03-25",
            "2025-03-14",
        )
        self.assertHolidayName(name, range(2001, 2036))
        dt = (
            "2004-03-08",
            "2007-03-05",
            "2011-03-21",
            "2017-03-13",
            "2021-03-29",
        )
        self.assertHolidayName(f"{name} (observed)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_deepavali_day(self):
        name = "Deepavali"
        self.assertHolidayName(
            name,
            "2020-11-14",
            "2021-11-04",
            "2022-10-24",
            "2023-11-12",
            "2024-10-31",
            "2025-10-20",
        )
        self.assertHolidayName(name, range(2001, 2036))
        dt = (
            "2013-11-04",
            "2016-10-31",
            "2019-10-28",
            "2023-11-13",
        )
        self.assertHolidayName(f"{name} (observed)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_youman_nabi_day(self):
        name = "Youman Nabi"
        self.assertHolidayName(
            name,
            "2020-10-29",
            "2021-10-19",
            "2022-10-09",
            "2023-09-28",
            "2024-09-16",
            "2025-09-05",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)
        dt = (
            "2004-05-03",
            "2012-02-06",
            "2019-11-11",
            "2022-10-10",
        )
        self.assertIslamicNoEstimatedHolidayName(f"{name} (observed)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_eid_ul_azha_day(self):
        name = "Eid-Ul-Azha"
        self.assertHolidayName(
            name,
            "2020-07-31",
            "2021-07-21",
            "2022-07-09",
            "2023-06-29",
            "2024-06-17",
            "2025-06-07",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)
        dt = (
            "2014-10-06",
            "2019-08-12",
        )
        self.assertIslamicNoEstimatedHolidayName(f"{name} (observed)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_2024(self):
        self.assertHolidaysInYear(
            2024,
            ("2024-01-01", "New Year's Day"),
            ("2024-02-23", "Republic Day"),
            ("2024-03-25", "Phagwah"),
            ("2024-03-29", "Good Friday"),
            ("2024-04-01", "Easter Monday"),
            ("2024-05-01", "Labour Day"),
            ("2024-05-05", "Arrival Day"),
            ("2024-05-06", "Arrival Day (observed)"),
            ("2024-05-26", "Independence Day"),
            ("2024-06-17", "Eid-Ul-Azha"),
            ("2024-07-01", "CARICOM Day"),
            ("2024-08-01", "Emancipation Day"),
            ("2024-09-16", "Youman Nabi"),
            ("2024-10-31", "Deepavali"),
            ("2024-12-25", "Christmas Day"),
            ("2024-12-26", "Day after Christmas"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2025-01-01", "New Year's Day"),
            ("2025-02-23", "Republic Day"),
            ("2025-03-14", "Phagwah"),
            ("2025-04-18", "Good Friday"),
            ("2025-04-21", "Easter Monday"),
            ("2025-05-01", "Labour Day"),
            ("2025-05-05", "Arrival Day"),
            ("2025-05-26", "Independence Day"),
            ("2025-06-07", "Eid-Ul-Azha"),
            ("2025-07-07", "CARICOM Day"),
            ("2025-08-01", "Emancipation Day"),
            ("2025-09-05", "Youman Nabi"),
            ("2025-10-20", "Deepavali"),
            ("2025-12-25", "Christmas Day"),
            ("2025-12-26", "Day after Christmas"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2025-01-01", "New Year's Day"),
            ("2025-02-23", "Republic Day"),
            ("2025-03-14", "Holi"),
            ("2025-04-18", "Good Friday"),
            ("2025-04-21", "Easter Monday"),
            ("2025-05-01", "Labor Day"),
            ("2025-05-05", "Arrival Day"),
            ("2025-05-26", "Independence Day"),
            ("2025-06-07", "Eid al-Adha"),
            ("2025-07-07", "CARICOM Day"),
            ("2025-08-01", "Emancipation Day"),
            ("2025-09-05", "Prophet's Birthday"),
            ("2025-10-20", "Diwali"),
            ("2025-12-25", "Christmas Day"),
            ("2025-12-26", "Day after Christmas"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.bahrain import Bahrain
from tests.common import CommonCountryTests


class TestBahrain(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Bahrain)

    def test_new_years_day(self):
        name = "رأس السنة الميلادية"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))

    def test_labor_day(self):
        name = "عيد العمال"
        self.assertHolidayName(name, (f"{year}-05-01" for year in self.full_range))

    def test_islamic_new_year(self):
        name = "رأس السنة الهجرية"
        self.assertHolidayName(
            name,
            "2020-08-20",
            "2021-08-09",
            "2022-07-30",
            "2023-07-19",
            "2024-07-07",
            "2025-06-26",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_ashura(self):
        name = "عاشوراء"
        self.assertHolidayName(
            name,
            "2020-08-28",
            "2020-08-29",
            "2021-08-17",
            "2021-08-18",
            "2022-08-07",
            "2022-08-08",
            "2023-07-28",
            "2023-07-29",
            "2024-07-15",
            "2024-07-16",
            "2025-07-04",
            "2025-07-05",
        )
        years_ashura_twice_all = {1976, 2009, 2042}
        self.assertIslamicNoEstimatedHolidayNameCount(
            name, 2, set(self.full_range) - years_ashura_twice_all
        )
        self.assertIslamicNoEstimatedHolidayNameCount(name, 4, years_ashura_twice_all)

    def test_prophets_birthday(self):
        name = "المولد النبوي الشريف"
        self.assertHolidayName(
            name,
            "2020-10-29",
            "2021-10-18",
            "2022-10-08",
            "2023-09-27",
            "2024-09-15",
            "2025-09-04",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_eid_al_fitr(self):
        name = "عيد الفطر"
        self.assertHolidayName(
            name,
            "2020-05-24",
            "2020-05-25",
            "2020-05-26",
            "2021-05-13",
            "2021-05-14",
            "2021-05-15",
            "2022-05-02",
            "2022-05-03",
            "2022-05-04",
            "2023-04-21",
            "2023-04-22",
            "2023-04-23",
            "2024-04-10",
            "2024-04-11",
            "2024-04-12",
            "2025-03-30",
            "2025-03-31",
            "2025-04-01",
        )
        years_eid_al_fitr_twice_all = {2000, 2033}
        self.assertIslamicNoEstimatedHolidayNameCount(
            name, 3, set(self.full_range) - years_eid_al_fitr_twice_all
        )
        self.assertIslamicNoEstimatedHolidayNameCount(name, 6, years_eid_al_fitr_twice_all)

    def test_eid_al_adha(self):
        name = "عيد الأضحى"
        self.assertHolidayName(
            name,
            "2020-07-31",
            "2020-08-01",
            "2020-08-02",
            "2021-07-20",
            "2021-07-21",
            "2021-07-22",
            "2022-07-09",
            "2022-07-10",
            "2022-07-11",
            "2023-06-28",
            "2023-06-29",
            "2023-06-30",
            "2024-06-16",
            "2024-06-17",
            "2024-06-18",
            "2025-06-06",
            "2025-06-07",
            "2025-06-08",
        )
        years_eid_al_adha_twice_all = {1974, 2039}
        self.assertIslamicNoEstimatedHolidayNameCount(
            name, 3, set(self.full_range) - years_eid_al_adha_twice_all - {2006, 2007}
        )
        self.assertIslamicNoEstimatedHolidayNameCount(name, 4, 2006)
        self.assertIslamicNoEstimatedHolidayNameCount(name, 5, 2007)
        self.assertIslamicNoEstimatedHolidayNameCount(name, 6, years_eid_al_adha_twice_all)

    def test_2022(self):
        self.assertHolidaysInYear(
            2022,
            ("2022-01-01", "رأس السنة الميلادية"),
            ("2022-05-01", "عيد العمال"),
            ("2022-05-02", "عيد الفطر"),
            ("2022-05-03", "عيد الفطر"),
            ("2022-05-04", "عيد الفطر"),
            ("2022-07-09", "عيد الأضحى"),
            ("2022-07-10", "عيد الأضحى"),
            ("2022-07-11", "عيد الأضحى"),
            ("2022-07-30", "رأس السنة الهجرية"),
            ("2022-08-07", "عاشوراء"),
            ("2022-08-08", "عاشوراء"),
            ("2022-10-08", "المولد النبوي الشريف"),
            ("2022-12-16", "العيد الوطني"),
            ("2022-12-17", "العيد الوطني"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2023-01-01", "رأس السنة الميلادية"),
            ("2023-04-21", "عيد الفطر"),
            ("2023-04-22", "عيد الفطر"),
            ("2023-04-23", "عيد الفطر"),
            ("2023-05-01", "عيد العمال"),
            ("2023-06-28", "عيد الأضحى"),
            ("2023-06-29", "عيد الأضحى"),
            ("2023-06-30", "عيد الأضحى"),
            ("2023-07-19", "رأس السنة الهجرية"),
            ("2023-07-28", "عاشوراء"),
            ("2023-07-29", "عاشوراء"),
            ("2023-09-27", "المولد النبوي الشريف"),
            ("2023-12-16", "العيد الوطني"),
            ("2023-12-17", "العيد الوطني"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2023-01-01", "New Year's Day"),
            ("2023-04-21", "Eid al-Fitr"),
            ("2023-04-22", "Eid al-Fitr"),
            ("2023-04-23", "Eid al-Fitr"),
            ("2023-05-01", "Labor Day"),
            ("2023-06-28", "Eid al-Adha"),
            ("2023-06-29", "Eid al-Adha"),
            ("2023-06-30", "Eid al-Adha"),
            ("2023-07-19", "Islamic New Year"),
            ("2023-07-28", "Ashura"),
            ("2023-07-29", "Ashura"),
            ("2023-09-27", "Prophet's Birthday"),
            ("2023-12-16", "National Day"),
            ("2023-12-17", "National Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.constants import (
    ALBANIAN,
    BOSNIAN,
    ISLAMIC,
    ROMA,
    SERBIAN,
    TURKISH,
    VLACH,
)
from holidays.countries.north_macedonia import NorthMacedonia
from tests.common import CommonCountryTests


class TestNorthMacedonia(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(NorthMacedonia)

    def test_no_holidays(self):
        super().test_no_holidays()

        self.assertNoHolidays(
            NorthMacedonia(
                categories=(ALBANIAN, BOSNIAN, ISLAMIC, ROMA, TURKISH, VLACH),
                years=range(self.start_year, 2007),
            )
        )
        self.assertNoHolidays(
            NorthMacedonia(categories=SERBIAN, years=range(self.start_year, 2008))
        )

    def test_special_holidays(self):
        self.assertHoliday(
            "2024-04-24",
            "2024-05-08",
        )

    def test_new_years_day(self):
        name = "Нова Година"
        self.assertHolidayName(
            name,
            (f"{year}-01-01" for year in self.full_range),
            (f"{year}-01-02" for year in range(self.start_year, 2008)),
        )
        self.assertNoNonObservedHolidayName(
            name, (f"{year}-01-02" for year in range(2008, self.end_year))
        )
        obs_dts = (
            "2000-01-03",
            "2005-01-03",
            "2006-01-02",
            "2012-01-02",
            "2017-01-02",
            "2023-01-02",
        )
        self.assertHolidayName(f"{name} (неработен ден)", obs_dts)

    def test_christmas_day(self):
        name = "Божиќ"
        self.assertHolidayName(name, (f"{year}-01-07" for year in range(2008, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2008))
        obs_dts = (
            "2018-01-08",
            "2024-01-08",
        )
        self.assertHolidayName(f"{name} (неработен ден)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_easter_monday(self):
        name = "Велигден"
        self.assertHolidayName(
            name,
            "2020-04-20",
            "2021-05-03",
            "2022-04-25",
            "2023-04-17",
            "2024-05-06",
            "2025-04-21",
        )
        self.assertHolidayName(name, range(2007, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 2007))

    def test_labor_day(self):
        name = "Ден на трудот"
        self.assertHolidayName(
            name,
            (f"{year}-05-01" for year in self.full_range),
            (f"{year}-05-02" for year in range(self.start_year, 2007)),
        )
        self.assertNoNonObservedHolidayName(
            name, (f"{year}-05-02" for year in range(2007, self.end_year))
        )
        obs_dts = (
            "1999-05-03",
            "2004-05-03",
            "2005-05-02",
            "2011-05-02",
            "2016-05-02",
            "2022-05-02",
        )
        self.assertHolidayName(f"{name} (неработен ден)", obs_dts)

    def test_saints_cyril_and_methodius_day(self):
        name = "Светите Кирил и Методиј - Ден на сесловенските просветители"
        self.assertHolidayName(name, (f"{year}-05-24" for year in range(2007, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2007))
        obs_dts = (
            "2009-05-25",
            "2015-05-25",
            "2020-05-25",
        )
        self.assertHolidayName(f"{name} (неработен ден)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_republic_day(self):
        name = "Ден на Републиката"
        self.assertHolidayName(name, (f"{year}-08-02" for year in self.full_range))
        obs_dts = (
            "2009-08-03",
            "2015-08-03",
            "2020-08-03",
        )
        self.assertHolidayName(f"{name} (неработен ден)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_independence_day(self):
        name = "Ден на независноста"
        self.assertHolidayName(name, (f"{year}-09-08" for year in self.full_range))
        obs_dts = (
            "2002-09-09",
            "2013-09-09",
            "2019-09-09",
            "2024-09-09",
        )
        self.assertHolidayName(f"{name} (неработен ден)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_national_uprising_day(self):
        name = "Ден на народното востание"
        self.assertHolidayName(name, (f"{year}-10-11" for year in self.full_range))
        obs_dts = (
            "2009-10-12",
            "2015-10-12",
            "2020-10-12",
        )
        self.assertHolidayName(f"{name} (неработен ден)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_revolutionary_struggle_day(self):
        name = "Ден на македонската револуционерна борба"
        self.assertHolidayName(name, (f"{year}-10-23" for year in range(2007, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2007))
        obs_dts = (
            "2011-10-24",
            "2016-10-24",
            "2022-10-24",
        )
        self.assertHolidayName(f"{name} (неработен ден)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_saint_clement_of_ohrid_day(self):
        name = "Свети Климент Охридски"
        self.assertHolidayName(name, (f"{year}-12-08" for year in range(2007, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2007))
        obs_dts = (
            "2013-12-09",
            "2019-12-09",
            "2024-12-09",
        )
        self.assertHolidayName(f"{name} (неработен ден)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_eid_al_fitr(self):
        name = "Рамазан Бајрам"
        self.assertHolidayName(
            name,
            "2020-05-24",
            "2021-05-13",
            "2022-05-02",
            "2023-04-21",
            "2024-04-10",
            "2025-03-30",
        )
        self.assertIslamicNoEstimatedHolidayName(name, range(2007, self.end_year))
        self.assertNoIslamicNoEstimatedHolidayName(name, range(self.start_year, 2007))
        obs_dts = (
            "2009-09-21",
            "2012-08-20",
            "2017-06-26",
            "2020-05-25",
            "2025-03-31",
        )
        self.assertIslamicNoEstimatedHolidayName(f"{name} (неработен ден)", obs_dts)

    def test_albanian_alphabet_day(self):
        name = "Ден на Албанската азбука"
        self.assertNoHolidayName(name)
        self.assertAlbanianHolidayName(
            name, (f"{year}-11-22" for year in range(2007, self.end_year))
        )
        self.assertNoAlbanianHolidayName(name, range(self.start_year, 2007))

    def test_international_bosniaks_day(self):
        name = "Меѓународен ден на Бошњаците"
        self.assertNoHolidayName(name)
        self.assertBosnianHolidayName(
            name, (f"{year}-09-28" for year in range(2007, self.end_year))
        )
        self.assertNoBosnianHolidayName(name, range(self.start_year, 2007))

    def test_easter_monday_catholic(self):
        name = "Велигден"
        self.assertCatholicHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertCatholicHolidayName(name, self.full_range)

    def test_all_saints_day_catholic(self):
        name = "Сите Светци"
        self.assertNoHolidayName(name)
        self.assertCatholicHolidayName(
            name, (f"{year}-11-01" for year in range(2007, self.end_year))
        )
        self.assertNoCatholicHolidayName(name, range(self.start_year, 2007))

    def test_christmas_day_catholic(self):
        self.assertCatholicHolidayName("Божиќ", (f"{year}-12-25" for year in self.full_range))

    def test_yom_kippur(self):
        name = "Јом Кипур"
        self.assertNoHolidayName(name)
        self.assertHebrewHolidayName(
            name,
            "2020-09-28",
            "2021-09-16",
            "2022-10-05",
            "2023-09-25",
            "2024-10-12",
            "2025-10-02",
        )
        self.assertHebrewHolidayName(name, self.full_range)

    def test_eid_al_adha_islamic(self):
        name = "Курбан Бајрам"
        self.assertNoHolidayName(name)
        self.assertIslamicHolidayName(
            name,
            "2020-07-31",
            "2021-07-20",
            "2022-07-09",
            "2023-06-28",
            "2024-06-16",
            "2025-06-06",
        )
        self.assertIslamicIslamicNoEstimatedHolidayName(name, range(2007, self.end_year))
        self.assertNoIslamicIslamicNoEstimatedHolidayName(name, range(self.start_year, 2007))

    def test_christmas_day_orthodox(self):
        name = "Божиќ"
        self.assertOrthodoxHolidayName(
            name, (f"{year}-01-07" for year in range(self.start_year, 2008))
        )
        self.assertNoOrthodoxHolidayName(name, range(2008, self.end_year))

    def test_easter_monday_orthodox(self):
        name = "Велигден"
        self.assertOrthodoxHolidayName(
            name,
            "1999-04-12",
            "2000-05-01",
            "2001-04-16",
            "2002-05-06",
            "2003-04-28",
            "2004-04-12",
            "2005-05-02",
            "2006-04-24",
        )
        self.assertNoOrthodoxHolidayName(name, range(2007, self.end_year))

    def test_christmas_eve_orthodox(self):
        name = "Бадник"
        self.assertNoHolidayName(name)
        self.assertOrthodoxHolidayName(
            name, (f"{year}-01-06" for year in range(2007, self.end_year))
        )
        self.assertNoOrthodoxHolidayName(name, range(self.start_year, 2007))

    def test_epiphany_orthodox(self):
        name = "Богојавление"
        self.assertNoHolidayName(name)
        self.assertOrthodoxHolidayName(
            name, (f"{year}-01-19" for year in range(2007, self.end_year))
        )
        self.assertNoOrthodoxHolidayName(name, range(self.start_year, 2007))

    def test_good_friday_orthodox(self):
        name = "Велики Петок"
        self.assertNoHolidayName(name)
        self.assertOrthodoxHolidayName(
            name,
            "2020-04-17",
            "2021-04-30",
            "2022-04-22",
            "2023-04-14",
            "2024-05-03",
            "2025-04-18",
        )
        self.assertOrthodoxHolidayName(name, range(2007, self.end_year))
        self.assertNoOrthodoxHolidayName(name, range(self.start_year, 2007))

    def test_assumption_of_mary_orthodox(self):
        name = "Успение на Пресвета Богородица"
        self.assertNoHolidayName(name)
        self.assertOrthodoxHolidayName(
            name, (f"{year}-08-28" for year in range(2007, self.end_year))
        )
        self.assertNoOrthodoxHolidayName(name, range(self.start_year, 2007))

    def test_pentecost_orthodox(self):
        name = "Духовден"
        self.assertNoHolidayName(name)
        self.assertOrthodoxHolidayName(
            name,
            "2020-06-05",
            "2021-06-18",
            "2022-06-10",
            "2023-06-02",
            "2024-06-21",
            "2025-06-06",
        )
        self.assertOrthodoxHolidayName(name, range(2007, self.end_year))
        self.assertNoOrthodoxHolidayName(name, range(self.start_year, 2007))

    def test_international_romani_day(self):
        name = "Меѓународен ден на Ромите"
        self.assertNoHolidayName(name)
        self.assertRomaHolidayName(name, (f"{year}-04-08" for year in range(2007, self.end_year)))
        self.assertNoRomaHolidayName(name, range(self.start_year, 2007))

    def test_saint_savas_day(self):
        name = "Свети Сава"
        self.assertNoHolidayName(name)
        self.assertSerbianHolidayName(
            name, (f"{year}-01-27" for year in range(2008, self.end_year))
        )
        self.assertNoSerbianHolidayName(name, range(self.start_year, 2008))

    def test_turkish_language_teaching_day(self):
        name = "Ден на настава на турски јазик"
        self.assertNoHolidayName(name)
        self.assertTurkishHolidayName(
            name, (f"{year}-12-21" for year in range(2007, self.end_year))
        )
        self.assertNoTurkishHolidayName(name, range(self.start_year, 2007))

    def test_vlachs_national_day(self):
        name = "Национален ден на Власите"
        self.assertNoHolidayName(name)
        self.assertVlachHolidayName(name, (f"{year}-05-23" for year in range(2007, self.end_year)))
        self.assertNoVlachHolidayName(name, range(self.start_year, 2007))

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2024-01-01", "Нова Година"),
            ("2024-01-06", "Бадник"),
            ("2024-01-07", "Божиќ"),
            ("2024-01-08", "Божиќ (неработен ден)"),
            ("2024-01-19", "Богојавление"),
            ("2024-01-27", "Свети Сава"),
            ("2024-04-01", "Велигден"),
            ("2024-04-08", "Меѓународен ден на Ромите"),
            ("2024-04-10", "Рамазан Бајрам"),
            ("2024-04-24", "Ден на изборите"),
            ("2024-05-01", "Ден на трудот"),
            ("2024-05-03", "Велики Петок"),
            ("2024-05-06", "Велигден"),
            ("2024-05-08", "Ден на изборите"),
            ("2024-05-23", "Национален ден на Власите"),
            ("2024-05-24", "Светите Кирил и Методиј - Ден на сесловенските просветители"),
            ("2024-06-16", "Курбан Бајрам"),
            ("2024-06-21", "Духовден"),
            ("2024-08-02", "Ден на Републиката"),
            ("2024-08-28", "Успение на Пресвета Богородица"),
            ("2024-09-08", "Ден на независноста"),
            ("2024-09-09", "Ден на независноста (неработен ден)"),
            ("2024-09-28", "Меѓународен ден на Бошњаците"),
            ("2024-10-11", "Ден на народното востание"),
            ("2024-10-12", "Јом Кипур"),
            ("2024-10-23", "Ден на македонската револуционерна борба"),
            ("2024-11-01", "Сите Светци"),
            ("2024-11-22", "Ден на Албанската азбука"),
            ("2024-12-08", "Свети Климент Охридски"),
            ("2024-12-09", "Свети Климент Охридски (неработен ден)"),
            ("2024-12-21", "Ден на настава на турски јазик"),
            ("2024-12-25", "Божиќ"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2024-01-01", "New Year's Day"),
            ("2024-01-06", "Christmas Eve"),
            ("2024-01-07", "Christmas Day"),
            ("2024-01-08", "Christmas Day (observed)"),
            ("2024-01-19", "Epiphany"),
            ("2024-01-27", "Saint Sava's Day"),
            ("2024-04-01", "Easter Monday"),
            ("2024-04-08", "International Romani Day"),
            ("2024-04-10", "Eid al-Fitr"),
            ("2024-04-24", "Election Day"),
            ("2024-05-01", "Labor Day"),
            ("2024-05-03", "Good Friday"),
            ("2024-05-06", "Easter Monday"),
            ("2024-05-08", "Election Day"),
            ("2024-05-23", "Vlachs National Day"),
            ("2024-05-24", "Saints Cyril and Methodius Day"),
            ("2024-06-16", "Eid al-Adha"),
            ("2024-06-21", "Pentecost"),
            ("2024-08-02", "Republic Day"),
            ("2024-08-28", "Dormition of the Mother of God"),
            ("2024-09-08", "Independence Day"),
            ("2024-09-09", "Independence Day (observed)"),
            ("2024-09-28", "International Bosniaks Day"),
            ("2024-10-11", "National Uprising Day"),
            ("2024-10-12", "Yom Kippur"),
            ("2024-10-23", "Macedonian Revolutionary Struggle Day"),
            ("2024-11-01", "All Saints' Day"),
            ("2024-11-22", "Albanian Alphabet Day"),
            ("2024-12-08", "Saint Clement of Ohrid Day"),
            ("2024-12-09", "Saint Clement of Ohrid Day (observed)"),
            ("2024-12-21", "Turkish Language Teaching Day"),
            ("2024-12-25", "Christmas Day"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2024-01-01", "Новий рік"),
            ("2024-01-06", "Святий вечір"),
            ("2024-01-07", "Різдво Христове"),
            ("2024-01-08", "Різдво Христове (вихідний)"),
            ("2024-01-19", "Богоявлення"),
            ("2024-01-27", "День Святого Сави"),
            ("2024-04-01", "Великодній понеділок"),
            ("2024-04-08", "Міжнародний день ромів"),
            ("2024-04-10", "Рамазан-байрам"),
            ("2024-04-24", "День виборів"),
            ("2024-05-01", "День праці"),
            ("2024-05-03", "Страсна пʼятниця"),
            ("2024-05-06", "Великодній понеділок"),
            ("2024-05-08", "День виборів"),
            ("2024-05-23", "Національний день влахів"),
            ("2024-05-24", "День Святих Кирила та Мефодія, всесловʼянських просвітителів"),
            ("2024-06-16", "Курбан-байрам"),
            ("2024-06-21", "Трійця"),
            ("2024-08-02", "День Республіки"),
            ("2024-08-28", "Успіння Пресвятої Богородиці"),
            ("2024-09-08", "День незалежності"),
            ("2024-09-09", "День незалежності (вихідний)"),
            ("2024-09-28", "Міжнародний день босняків"),
            ("2024-10-11", "День народного повстання"),
            ("2024-10-12", "Йом Кіпур"),
            ("2024-10-23", "День македонської революційної боротьби"),
            ("2024-11-01", "День усіх святих"),
            ("2024-11-22", "День албанського алфавіту"),
            ("2024-12-08", "День Святого Климента Охридського"),
            ("2024-12-09", "День Святого Климента Охридського (вихідний)"),
            ("2024-12-21", "День викладання турецької мови"),
            ("2024-12-25", "Різдво Христове"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.helpers import _normalize_arguments, _normalize_tuple


class TestHelpers(TestCase):
    def test_normalize_arguments(self):
        empty_set = set()
        input_expected_pairs = (
            ((int, []), empty_set),
            ((int, {}), empty_set),
            ((int, None), empty_set),
            ((str, []), empty_set),
            ((str, {}), empty_set),
            ((str, None), empty_set),
            ((int, "1"), {1}),
            ((int, "12"), {12}),
            ((int, "121"), {121}),
            ((int, ["1", "1"]), {1}),
            ((int, ["1", "2"]), {1, 2}),
            ((int, [1, 1]), {1}),
            ((int, [1, 2]), {1, 2}),
            ((int, [1]), {1}),
            ((int, {1}), {1}),
            ((int, 0), {0}),
            ((int, 1.0), {1.0}),
            ((int, 1), {1}),
            ((str, "1"), {"1"}),
            ((str, ("test1", "TEST1")), {"test1", "TEST1"}),
            ((str, ("test1", "test1")), {"test1"}),
            ((str, ("test1", "test2")), {"test1", "test2"}),
            ((str, ["1", "2"]), {"1", "2"}),
            ((str, [1, 2]), {"1", "2"}),
            ((str, [1]), {"1"}),
            ((str, {1: "2"}), {"1"}),
            ((str, {1: 2}), {"1"}),
            ((str, {1}), {"1"}),
            ((str, 1.0), {"1.0"}),
            ((str, 1), {"1"}),
        )

        for input, expected in input_expected_pairs:
            self.assertEqual(_normalize_arguments(*input), expected)

    def test_normalize_tuple(self):
        input_expected_pairs = (
            ((), ()),
            ((1,), ((1,),)),
            ((1, 2), ((1, 2),)),
            (((1, 1), (2, 2)), ((1, 1), (2, 2))),
        )
        for input, expected in input_expected_pairs:
            self.assertEqual(_normalize_tuple(input), expected)



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.groups import BurmeseCalendarHolidays
from holidays.holiday_base import HolidayBase


class TestBurmeseCalendarHolidays(TestCase):
    def test_add_burmese_calendar_holiday(self):
        # Check for out-of-range dates.
        class TestHolidays(HolidayBase, BurmeseCalendarHolidays):
            end_year = 2101

            def __init__(self, *args, **kwargs):
                BurmeseCalendarHolidays.__init__(self)
                super().__init__(*args, **kwargs)

        test_holidays = TestHolidays()

        test_holidays._populate(2101)
        test_holidays._add_myanmar_new_year("Myanmar New Year")
        test_holidays._add_tabaung_full_moon_day("Full Moon Day of Tabaung")
        test_holidays._add_kason_full_moon_day("Full Moon Day of Kason")
        test_holidays._add_waso_full_moon_day("Full Moon Day of Waso")
        test_holidays._add_thadingyut_full_moon_day("Thadingyut Holiday")
        test_holidays._add_myanmar_diwali("Diwali")
        test_holidays._add_tazaungmon_full_moon_day("Full Moon Day of Tazaungmon")
        test_holidays._add_myanmar_national_day("National Day")
        test_holidays._add_karen_new_year("Karen New Year")

        self.assertEqual(0, len(test_holidays))



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.constants import WORKDAY
from holidays.countries.slovakia import Slovakia
from tests.common import CommonCountryTests


class TestSlovakia(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Slovakia)

    def test_no_holidays(self):
        super().test_no_holidays()

        self.assertNoHolidays(Slovakia(categories=WORKDAY, years=range(self.start_year, 2021)))

    def test_special_holidays(self):
        self.assertHoliday(
            "2018-10-30",
        )

    def test_day_of_the_establishment_of_the_slovak_republic(self):
        self.assertHolidayName(
            "Deň vzniku Slovenskej republiky", (f"{year}-01-01" for year in self.full_range)
        )

    def test_epiphany(self):
        self.assertHolidayName(
            "Zjavenie Pána (Traja králi a vianočný sviatok pravoslávnych kresťanov)",
            (f"{year}-01-06" for year in self.full_range),
        )

    def test_good_friday(self):
        name = "Veľký piatok"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "Veľkonočný pondelok"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_labor_day(self):
        self.assertHolidayName("Sviatok práce", (f"{year}-05-01" for year in self.full_range))

    def test_day_of_victory_over_fascism(self):
        name = "Deň víťazstva nad fašizmom"
        self.assertHolidayName(
            name, (f"{year}-05-08" for year in (*range(1997, 2026), *range(2027, self.end_year)))
        )
        self.assertNoHolidayName(name, range(self.start_year, 1997), 2026)
        self.assertWorkdayHolidayName(name, "2026-05-08")
        self.assertNoWorkdayHolidayName(
            name, range(self.start_year, 2026), range(2027, self.end_year)
        )

    def test_saints_cyril_and_methodius_day(self):
        self.assertHolidayName(
            "Sviatok svätého Cyrila a svätého Metoda",
            (f"{year}-07-05" for year in self.full_range),
        )

    def test_slovak_national_uprising_anniversary(self):
        self.assertHolidayName(
            "Výročie Slovenského národného povstania",
            (f"{year}-08-29" for year in self.full_range),
        )

    def test_constitution_day(self):
        name = "Deň Ústavy Slovenskej republiky"
        # PUBLIC.
        self.assertHolidayName(name, (f"{year}-09-01" for year in range(self.start_year, 2024)))
        self.assertNoHolidayName(name, range(2024, self.end_year))
        # WORKDAY.
        self.assertWorkdayHolidayName(
            name, (f"{year}-09-01" for year in range(2024, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 2024))

    def test_day_of_our_lady_of_the_seven_sorrows(self):
        name = "Sedembolestná Panna Mária"
        self.assertHolidayName(
            name,
            (
                f"{year}-09-15"
                for year in (*range(self.start_year, 2026), *range(2027, self.end_year))
            ),
        )
        self.assertNoHolidayName(name, 2026)
        self.assertWorkdayHolidayName(name, "2026-09-15")
        self.assertNoWorkdayHolidayName(
            name, range(self.start_year, 2026), range(2027, self.end_year)
        )

    def test_day_of_the_establishment_of_the_czech_slovak_state(self):
        name = "Deň vzniku samostatného česko-slovenského štátu"
        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(
            name, (f"{year}-10-28" for year in range(2021, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 2021))

    def test_all_saints_day(self):
        self.assertHolidayName(
            "Sviatok Všetkých svätých", (f"{year}-11-01" for year in self.full_range)
        )

    def test_struggle_for_freedom_and_democracy_day(self):
        name = "Deň boja za slobodu a demokraciu"
        # PUBLIC.
        self.assertHolidayName(name, (f"{year}-11-17" for year in range(2001, 2025)))
        self.assertNoHolidayName(name, range(self.start_year, 2001), range(2025, self.end_year))
        # WORKDAY.
        self.assertWorkdayHolidayName(
            name, (f"{year}-11-17" for year in range(2025, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 2025))

    def test_christmas_eve(self):
        self.assertHolidayName("Štedrý deň", (f"{year}-12-24" for year in self.full_range))

    def test_christmas_day(self):
        self.assertHolidayName(
            "Prvý sviatok vianočný", (f"{year}-12-25" for year in self.full_range)
        )

    def test_second_day_of_christmas(self):
        self.assertHolidayName(
            "Druhý sviatok vianočný", (f"{year}-12-26" for year in self.full_range)
        )

    def test_2021_public(self):
        self.assertHolidaysInYear(
            2021,
            ("2021-01-01", "Deň vzniku Slovenskej republiky"),
            (
                "2021-01-06",
                "Zjavenie Pána (Traja králi a vianočný sviatok pravoslávnych kresťanov)",
            ),
            ("2021-04-02", "Veľký piatok"),
            ("2021-04-05", "Veľkonočný pondelok"),
            ("2021-05-01", "Sviatok práce"),
            ("2021-05-08", "Deň víťazstva nad fašizmom"),
            ("2021-07-05", "Sviatok svätého Cyrila a svätého Metoda"),
            ("2021-08-29", "Výročie Slovenského národného povstania"),
            ("2021-09-01", "Deň Ústavy Slovenskej republiky"),
            ("2021-09-15", "Sedembolestná Panna Mária"),
            ("2021-11-01", "Sviatok Všetkých svätých"),
            ("2021-11-17", "Deň boja za slobodu a demokraciu"),
            ("2021-12-24", "Štedrý deň"),
            ("2021-12-25", "Prvý sviatok vianočný"),
            ("2021-12-26", "Druhý sviatok vianočný"),
        )

    def test_2021_workday(self):
        self.assertWorkdayHolidaysInYear(
            2021,
            ("2021-10-28", "Deň vzniku samostatného česko-slovenského štátu"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "Deň vzniku Slovenskej republiky"),
            (
                "2022-01-06",
                "Zjavenie Pána (Traja králi a vianočný sviatok pravoslávnych kresťanov)",
            ),
            ("2022-04-15", "Veľký piatok"),
            ("2022-04-18", "Veľkonočný pondelok"),
            ("2022-05-01", "Sviatok práce"),
            ("2022-05-08", "Deň víťazstva nad fašizmom"),
            ("2022-07-05", "Sviatok svätého Cyrila a svätého Metoda"),
            ("2022-08-29", "Výročie Slovenského národného povstania"),
            ("2022-09-01", "Deň Ústavy Slovenskej republiky"),
            ("2022-09-15", "Sedembolestná Panna Mária"),
            ("2022-10-28", "Deň vzniku samostatného česko-slovenského štátu"),
            ("2022-11-01", "Sviatok Všetkých svätých"),
            ("2022-11-17", "Deň boja za slobodu a demokraciu"),
            ("2022-12-24", "Štedrý deň"),
            ("2022-12-25", "Prvý sviatok vianočný"),
            ("2022-12-26", "Druhý sviatok vianočný"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "Day of the Establishment of the Slovak Republic"),
            ("2022-01-06", "Epiphany (Three Kings' Day and Orthodox Christmas)"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-05-01", "Labor Day"),
            ("2022-05-08", "Day of Victory over Fascism"),
            ("2022-07-05", "Saints Cyril and Methodius Day"),
            ("2022-08-29", "Slovak National Uprising Anniversary"),
            ("2022-09-01", "Constitution Day"),
            ("2022-09-15", "Day of Our Lady of the Seven Sorrows"),
            ("2022-10-28", "Day of the Establishment of the Independent Czech-Slovak State"),
            ("2022-11-01", "All Saints' Day"),
            ("2022-11-17", "Struggle for Freedom and Democracy Day"),
            ("2022-12-24", "Christmas Eve"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Second Day of Christmas"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2022-01-01", "День утворення Словацької Республіки"),
            ("2022-01-06", "Богоявлення (Три царі і православне Різдво Христове)"),
            ("2022-04-15", "Страсна пʼятниця"),
            ("2022-04-18", "Великодній понеділок"),
            ("2022-05-01", "День праці"),
            ("2022-05-08", "День перемоги над фашизмом"),
            ("2022-07-05", "День Святих Кирила та Мефодія"),
            ("2022-08-29", "Річниця Словацького національного повстання"),
            ("2022-09-01", "День конституції Словацької Республіки"),
            ("2022-09-15", "День Божої Матері семи скорбот"),
            ("2022-10-28", "День створення незалежної чесько-словацької держави"),
            ("2022-11-01", "День усіх святих"),
            ("2022-11-17", "День боротьби за свободу та демократію"),
            ("2022-12-24", "Святий вечір"),
            ("2022-12-25", "Різдво Христове"),
            ("2022-12-26", "Другий день Різдва"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.sint_maarten import SintMaarten
from tests.common import CommonCountryTests


class TestSintMaarten(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(SintMaarten, years=range(2011, 2050))

    def test_new_years_day(self):
        self.assertHolidayName("Nieuwjaarsdag", (f"{year}-01-01" for year in range(2011, 2050)))

    def test_good_friday(self):
        name = "Goede Vrijdag"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, range(2011, 2050))

    def test_easter_sunday(self):
        name = "Eerste paasdag"
        self.assertHolidayName(
            name,
            "2020-04-12",
            "2021-04-04",
            "2022-04-17",
            "2023-04-09",
            "2024-03-31",
            "2025-04-20",
        )
        self.assertHolidayName(name, range(2011, 2050))

    def test_easter_monday(self):
        name = "Tweede paasdag"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, range(2011, 2050))

    def test_kings_day(self):
        name_queen = "Koninginnedag"
        name_king = "Koningsdag"
        self.assertHolidayName(name_queen, (f"{year}-04-30" for year in range(2011, 2014)))
        self.assertNoHolidayName(name_queen, range(2014, 2050))
        self.assertHolidayName(
            name_king,
            "2014-04-26",
            "2015-04-27",
            "2020-04-27",
            "2021-04-27",
            "2022-04-27",
            "2023-04-27",
            "2024-04-27",
            "2025-04-26",
        )
        self.assertNoHoliday(
            "2014-04-27",
            "2025-04-27",
        )
        self.assertHolidayName(name_king, range(2014, 2050))
        self.assertNoHolidayName(name_king, range(2011, 2014))

    def test_carnival_day(self):
        name = "Carnavalsdag"
        self.assertHolidayName(name, (f"{year}-04-29" for year in range(2011, 2014)))
        self.assertHolidayName(
            name,
            "2020-04-30",
            "2021-04-30",
            "2022-04-29",
            "2023-05-02",
            "2024-04-30",
            "2025-04-30",
        )
        self.assertNoHoliday(
            "2016-04-30",
            "2017-04-30",
            "2022-04-30",
            "2023-04-30",
        )
        self.assertHolidayName(name, range(2011, 2050))

    def test_labor_day(self):
        self.assertHolidayName(
            "Dag van de Arbeid", (f"{year}-05-01" for year in range(2011, 2050))
        )

    def test_ascension_day(self):
        name = "Hemelvaartsdag"
        self.assertHolidayName(
            name,
            "2020-05-21",
            "2021-05-13",
            "2022-05-26",
            "2023-05-18",
            "2024-05-09",
            "2025-05-29",
        )
        self.assertHolidayName(name, range(2011, 2050))

    def test_whit_sunday(self):
        name = "Eerste Pinksterdag"
        self.assertHolidayName(
            name,
            "2020-05-31",
            "2021-05-23",
            "2022-06-05",
            "2023-05-28",
            "2024-05-19",
            "2025-06-08",
        )
        self.assertHolidayName(name, range(2011, 2050))

    def test_emancipation_day(self):
        name = "Dag van de Bevrijding"
        self.assertHolidayName(
            name,
            "2018-07-02",
            "2019-07-01",
            "2020-07-01",
            "2021-07-01",
            "2022-07-01",
            "2023-07-01",
            "2024-07-01",
            "2025-07-01",
        )
        self.assertNoHoliday(
            "2012-08-01",
            "2018-08-01",
        )
        self.assertHolidayName(name, range(2012, 2050))
        self.assertNoHolidayName(name, 2011)

    def test_constitution_day(self):
        name = "Dag van de Constitutie"
        self.assertHolidayName(
            name,
            "2020-10-12",
            "2021-10-11",
            "2022-10-10",
            "2023-10-09",
            "2024-10-14",
            "2025-10-13",
        )
        self.assertHolidayName(name, range(2015, 2050))
        self.assertNoHolidayName(name, range(2011, 2015))

    def test_sint_maarten_day(self):
        self.assertHolidayName("Sint-Maartensdag", (f"{year}-11-11" for year in range(2011, 2050)))

    def test_kingdom_day(self):
        name = "Koninkrijksdag"
        self.assertHolidayName(
            name,
            "2011-12-15",
            "2012-12-15",
            "2013-12-16",
            "2014-12-15",
        )
        self.assertNoHolidayName(name, range(2015, 2050))

    def test_christmas(self):
        self.assertHolidayName("Eerste Kerstdag", (f"{year}-12-25" for year in range(2011, 2050)))
        self.assertHolidayName("Tweede Kerstdag", (f"{year}-12-26" for year in range(2011, 2050)))

    def test_2017(self):
        self.assertHolidaysInYear(
            2017,
            ("2017-01-01", "Nieuwjaarsdag"),
            ("2017-04-14", "Goede Vrijdag"),
            ("2017-04-16", "Eerste paasdag"),
            ("2017-04-17", "Tweede paasdag"),
            ("2017-04-27", "Koningsdag"),
            ("2017-05-02", "Carnavalsdag"),
            ("2017-05-01", "Dag van de Arbeid"),
            ("2017-05-25", "Hemelvaartsdag"),
            ("2017-06-04", "Eerste Pinksterdag"),
            ("2017-07-01", "Dag van de Bevrijding"),
            ("2017-10-09", "Dag van de Constitutie"),
            ("2017-11-11", "Sint-Maartensdag"),
            ("2017-12-25", "Eerste Kerstdag"),
            ("2017-12-26", "Tweede Kerstdag"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "Nieuwjaarsdag"),
            ("2022-04-15", "Goede Vrijdag"),
            ("2022-04-17", "Eerste paasdag"),
            ("2022-04-18", "Tweede paasdag"),
            ("2022-04-27", "Koningsdag"),
            ("2022-04-29", "Carnavalsdag"),
            ("2022-05-01", "Dag van de Arbeid"),
            ("2022-05-26", "Hemelvaartsdag"),
            ("2022-06-05", "Eerste Pinksterdag"),
            ("2022-07-01", "Dag van de Bevrijding"),
            ("2022-10-10", "Dag van de Constitutie"),
            ("2022-11-11", "Sint-Maartensdag"),
            ("2022-12-25", "Eerste Kerstdag"),
            ("2022-12-26", "Tweede Kerstdag"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-17", "Easter Sunday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-04-27", "King's Day"),
            ("2022-04-29", "Carnival Day"),
            ("2022-05-01", "Labor Day"),
            ("2022-05-26", "Ascension Day"),
            ("2022-06-05", "Whit Sunday"),
            ("2022-07-01", "Emancipation Day"),
            ("2022-10-10", "Constitution Day"),
            ("2022-11-11", "Sint Maarten Day"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Second Day of Christmas"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.el_salvador import ElSalvador
from tests.common import CommonCountryTests


class TestElSalvador(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(ElSalvador)

    def test_new_years_day(self):
        self.assertHolidayName("Año Nuevo", (f"{year}-01-01" for year in self.full_range))

    def test_maundy_thursday(self):
        name = "Jueves Santo"
        self.assertHolidayName(
            name,
            "2020-04-09",
            "2021-04-01",
            "2022-04-14",
            "2023-04-06",
            "2024-03-28",
            "2025-04-17",
        )
        self.assertHolidayName(name, self.full_range)

    def test_good_friday(self):
        name = "Viernes Santo"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_holy_saturday(self):
        name = "Sábado Santo"
        self.assertHolidayName(
            name,
            "2020-04-11",
            "2021-04-03",
            "2022-04-16",
            "2023-04-08",
            "2024-03-30",
            "2025-04-19",
        )
        self.assertHolidayName(name, self.full_range)

    def test_labor_day(self):
        self.assertHolidayName("Día del Trabajo", (f"{year}-05-01" for year in self.full_range))

    def test_mothers_day(self):
        name = "Día de la Madre"
        self.assertHolidayName(name, (f"{year}-05-10" for year in range(2016, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2016))

    def test_fathers_day(self):
        name = "Día del Padre"
        self.assertHolidayName(name, (f"{year}-06-17" for year in range(2013, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2013))

    def test_celebrations_of_san_salvador(self):
        self.assertHolidayName(
            "Celebración del Divino Salvador del Mundo",
            (f"{year}-08-06" for year in self.full_range),
        )

    def test_independence_day(self):
        self.assertHolidayName(
            "Día de la Independencia", (f"{year}-09-15" for year in self.full_range)
        )

    def test_all_souls_day(self):
        self.assertHolidayName(
            "Día de los Difuntos", (f"{year}-11-02" for year in self.full_range)
        )

    def test_christmas_day(self):
        self.assertHolidayName("Navidad", (f"{year}-12-25" for year in self.full_range))

    def test_feast_of_san_salvador(self):
        name = "Fiesta de San Salvador"
        self.assertNoHolidayName(name)
        self.assertSubdivSsHolidayName(
            name,
            (f"{year}-08-03" for year in self.full_range),
            (f"{year}-08-05" for year in self.full_range),
        )

    def test_2021(self):
        self.assertHolidaysInYear(
            2021,
            ("2021-01-01", "Año Nuevo"),
            ("2021-04-01", "Jueves Santo"),
            ("2021-04-02", "Viernes Santo"),
            ("2021-04-03", "Sábado Santo"),
            ("2021-05-01", "Día del Trabajo"),
            ("2021-05-10", "Día de la Madre"),
            ("2021-06-17", "Día del Padre"),
            ("2021-08-06", "Celebración del Divino Salvador del Mundo"),
            ("2021-09-15", "Día de la Independencia"),
            ("2021-11-02", "Día de los Difuntos"),
            ("2021-12-25", "Navidad"),
        )

    def test_2022(self):
        self.assertHolidaysInYear(
            2022,
            ("2022-01-01", "Año Nuevo"),
            ("2022-04-14", "Jueves Santo"),
            ("2022-04-15", "Viernes Santo"),
            ("2022-04-16", "Sábado Santo"),
            ("2022-05-01", "Día del Trabajo"),
            ("2022-05-10", "Día de la Madre"),
            ("2022-06-17", "Día del Padre"),
            ("2022-08-06", "Celebración del Divino Salvador del Mundo"),
            ("2022-09-15", "Día de la Independencia"),
            ("2022-11-02", "Día de los Difuntos"),
            ("2022-12-25", "Navidad"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2024-01-01", "Año Nuevo"),
            ("2024-03-28", "Jueves Santo"),
            ("2024-03-29", "Viernes Santo"),
            ("2024-03-30", "Sábado Santo"),
            ("2024-05-01", "Día del Trabajo"),
            ("2024-05-10", "Día de la Madre"),
            ("2024-06-17", "Día del Padre"),
            ("2024-08-03", "Fiesta de San Salvador"),
            ("2024-08-05", "Fiesta de San Salvador"),
            ("2024-08-06", "Celebración del Divino Salvador del Mundo"),
            ("2024-09-15", "Día de la Independencia"),
            ("2024-11-02", "Día de los Difuntos"),
            ("2024-12-25", "Navidad"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2024-01-01", "New Year's Day"),
            ("2024-03-28", "Maundy Thursday"),
            ("2024-03-29", "Good Friday"),
            ("2024-03-30", "Holy Saturday"),
            ("2024-05-01", "Labor Day"),
            ("2024-05-10", "Mother's Day"),
            ("2024-06-17", "Father's Day"),
            ("2024-08-03", "Feast of San Salvador"),
            ("2024-08-05", "Feast of San Salvador"),
            ("2024-08-06", "Celebrations of San Salvador"),
            ("2024-09-15", "Independence Day"),
            ("2024-11-02", "All Souls' Day"),
            ("2024-12-25", "Christmas Day"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2024-01-01", "Новий рік"),
            ("2024-03-28", "Великий четвер"),
            ("2024-03-29", "Страсна пʼятниця"),
            ("2024-03-30", "Велика субота"),
            ("2024-05-01", "День праці"),
            ("2024-05-10", "День матері"),
            ("2024-06-17", "День батька"),
            ("2024-08-03", "Свято Спасителя"),
            ("2024-08-05", "Свято Спасителя"),
            ("2024-08-06", "Свято Божественного Спасителя світу"),
            ("2024-09-15", "День незалежності"),
            ("2024-11-02", "День усіх померлих"),
            ("2024-12-25", "Різдво Христове"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.central_african_republic import CentralAfricanRepublic
from tests.common import CommonCountryTests


class TestCentralAfricanRepublic(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(CentralAfricanRepublic)

    def test_new_years_day(self):
        self.assertHolidayName("Jour de l'an", (f"{year}-01-01" for year in self.full_range))

    def test_barthelemy_boganda_day(self):
        name = "Journée Barthélemy Boganda"
        self.assertHolidayName(name, (f"{year}-03-29" for year in range(1960, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1960))

    def test_easter_monday(self):
        name = "Lundi de Pâques"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_labor_day(self):
        self.assertHolidayName("Fête du Travail", (f"{year}-05-01" for year in self.full_range))

    def test_ascension_day(self):
        name = "Ascension"
        self.assertHolidayName(
            name,
            "2020-05-21",
            "2021-05-13",
            "2022-05-26",
            "2023-05-18",
            "2024-05-09",
            "2025-05-29",
        )
        self.assertHolidayName(name, self.full_range)

    def test_whit_monday(self):
        name = "Lundi de Pentecôte"
        self.assertHolidayName(
            name,
            "2020-06-01",
            "2021-05-24",
            "2022-06-06",
            "2023-05-29",
            "2024-05-20",
            "2025-06-09",
        )
        self.assertHolidayName(name, self.full_range)

    def test_general_prayer_day(self):
        name = "Journée de prière générale"
        self.assertHolidayName(name, (f"{year}-06-30" for year in range(2007, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2007))

    def test_independence_day(self):
        name = "Jour de l'indépendance"
        self.assertHolidayName(name, (f"{year}-08-13" for year in range(1960, self.end_year)))
        self.assertNoHolidayName(name, self.start_year)

    def test_assumption_day(self):
        self.assertHolidayName("Assomption", (f"{year}-08-15" for year in self.full_range))

    def test_all_saints_day(self):
        self.assertHolidayName("Toussaint", (f"{year}-11-01" for year in self.full_range))

    def test_national_day(self):
        name = "Fête nationale"
        self.assertHolidayName(
            name,
            (
                f"{year}-12-01"
                for year in (*range(self.start_year, 1977), *range(1979, self.end_year))
            ),
        )
        self.assertHolidayName(
            name,
            "1977-12-04",
            "1978-12-04",
        )

    def test_christmas_day(self):
        self.assertHolidayName("Jour de Noël", (f"{year}-12-25" for year in self.full_range))

    def test_eid_al_fitr(self):
        name = "Aïd al-Fitr"
        self.assertHolidayName(
            name,
            "2020-05-24",
            "2021-05-13",
            "2022-05-02",
            "2023-04-21",
            "2024-04-10",
            "2025-03-30",
        )
        self.assertIslamicNoEstimatedHolidayName(name, range(2015, self.end_year))
        self.assertNoIslamicNoEstimatedHolidayName(name, range(self.start_year, 2015))

    def test_eid_al_adha(self):
        name = "Aïd al-Adha"
        self.assertHolidayName(
            name,
            "2020-07-31",
            "2021-07-20",
            "2022-07-09",
            "2023-06-28",
            "2024-06-16",
            "2025-06-07",
        )
        self.assertIslamicNoEstimatedHolidayName(name, range(2015, self.end_year))
        self.assertNoIslamicNoEstimatedHolidayName(name, range(self.start_year, 2015))

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2019-01-01", "Jour de l'an"),
            ("2019-03-29", "Journée Barthélemy Boganda"),
            ("2019-04-22", "Lundi de Pâques"),
            ("2019-05-01", "Fête du Travail"),
            ("2019-05-30", "Ascension"),
            ("2019-06-04", "Aïd al-Fitr"),
            ("2019-06-10", "Lundi de Pentecôte"),
            ("2019-06-30", "Journée de prière générale"),
            ("2019-08-11", "Aïd al-Adha"),
            ("2019-08-13", "Jour de l'indépendance"),
            ("2019-08-15", "Assomption"),
            ("2019-11-01", "Toussaint"),
            ("2019-12-01", "Fête nationale"),
            ("2019-12-25", "Jour de Noël"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2019-01-01", "New Year's Day"),
            ("2019-03-29", "Barthélemy Boganda Day"),
            ("2019-04-22", "Easter Monday"),
            ("2019-05-01", "Labor Day"),
            ("2019-05-30", "Ascension Day"),
            ("2019-06-04", "Eid al-Fitr"),
            ("2019-06-10", "Whit Monday"),
            ("2019-06-30", "General Prayer Day"),
            ("2019-08-11", "Eid al-Adha"),
            ("2019-08-13", "Independence Day"),
            ("2019-08-15", "Assumption Day"),
            ("2019-11-01", "All Saints' Day"),
            ("2019-12-01", "National Day"),
            ("2019-12-25", "Christmas Day"),
        )
